@@ -14,25 +14,28 @@
 
 #define cm_TGTLifeTime(x)	(0x7fffffff)
 
+#define CM_ACLENT_MAGIC    ('A' | 'C' <<8 | 'L'<<16 | 'E'<<24)
+
 /*
  * Structure to hold an acl entry for a cached file
  */
 typedef struct cm_aclent {
     osi_queue_t q;		/* for quick removal from LRUQ */
+    afs_uint32 magic;           
     struct cm_aclent *nextp;	/* next guy same vnode */
     struct cm_scache *backp;	/* back ptr to vnode */
     struct cm_user *userp;	/* user whose access is cached */
     long randomAccess;		/* watch for more rights in acl.h */
-    long tgtLifetime;		/* time this expires */
+    unsigned long tgtLifetime;	/* time this expires */
 } cm_aclent_t;
 
 extern osi_rwlock_t cm_aclLock;
 
-extern long cm_InitACLCache(long size);
+extern long cm_InitACLCache(int newFile, long size);
 
 extern long cm_FindACLCache(struct cm_scache *scp, struct cm_user *userp, long *rightsp);
 
-static cm_aclent_t *GetFreeACLEnt(void);
+static cm_aclent_t *GetFreeACLEnt(cm_scache_t * scp);
 
 extern long cm_AddACLCache(struct cm_scache *scp, struct cm_user *userp, long rights);
 
@@ -40,4 +43,8 @@ extern void cm_FreeAllACLEnts(struct cm_scache *scp);
 
 extern void cm_InvalidateACLUser(struct cm_scache *scp, struct cm_user *userp);
 
+extern long cm_ValidateACLCache(void);
+
+extern long cm_ShutdownACLCache(void);
+
 #endif  /* _CM_ACLENT_H_ */
