@@ -14,6 +14,7 @@
 
 #include <rx/rx.h>
 
+#include <WINNT/afsreg.h>
 #include "afsd.h"
 #ifdef AFS_FREELANCE_CLIENT
 #include "cm_freelance.h"
@@ -25,22 +26,26 @@ int cm_noLocalMountPoints;
 int cm_fakeDirSize;
 int cm_fakeDirCallback=0;
 int cm_fakeGettingCallback=0;
-int cm_fakeDirVersion = 0x8;
 cm_localMountPoint_t* cm_localMountPoints;
 osi_mutex_t cm_Freelance_Lock;
 int cm_localMountPointChangeFlag = 0;
 int cm_freelanceEnabled = 0;
 time_t FakeFreelanceModTime = 0x3b49f6e2;
 
+static int freelance_ShutdownFlag = 0;
+#if !defined(DJGPP)
+static HANDLE hFreelanceChangeEvent = 0;
+static HANDLE hFreelanceSymlinkChangeEvent = 0;
+#endif
+
 void cm_InitFakeRootDir();
 
 #if !defined(DJGPP)
 void cm_FreelanceChangeNotifier(void * parmp) {
-    HANDLE hFreelanceChangeEvent = 0;
     HKEY   hkFreelance = 0;
 
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_NOTIFY,
                       &hkFreelance) == ERROR_SUCCESS) {
@@ -64,22 +69,28 @@ void cm_FreelanceChangeNotifier(void * parmp) {
                                      ) != ERROR_SUCCESS) {
             RegCloseKey(hkFreelance);
             CloseHandle(hFreelanceChangeEvent);
+            hFreelanceChangeEvent = 0;
             return;
         }
 
         if (WaitForSingleObject(hFreelanceChangeEvent, INFINITE) == WAIT_OBJECT_0)
         {
+            if (freelance_ShutdownFlag == 1) {     
+                RegCloseKey(hkFreelance);          
+                CloseHandle(hFreelanceChangeEvent);
+                hFreelanceChangeEvent = 0;         
+                return;                            
+            }                                      
             cm_noteLocalMountPointChange();
         }
     }
 }
 
 void cm_FreelanceSymlinkChangeNotifier(void * parmp) {
-    HANDLE hFreelanceSymlinkChangeEvent = 0;
     HKEY   hkFreelance = 0;
 
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance\\Symlinks",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                       0,
                       KEY_NOTIFY,
                       &hkFreelance) == ERROR_SUCCESS) {
@@ -103,17 +114,36 @@ void cm_FreelanceSymlinkChangeNotifier(void * parmp) {
                                      ) != ERROR_SUCCESS) {
             RegCloseKey(hkFreelance);
             CloseHandle(hFreelanceSymlinkChangeEvent);
+            hFreelanceSymlinkChangeEvent = 0;
             return;
         }
 
         if (WaitForSingleObject(hFreelanceSymlinkChangeEvent, INFINITE) == WAIT_OBJECT_0)
         {
+            if (freelance_ShutdownFlag == 1) {     
+                RegCloseKey(hkFreelance);          
+                CloseHandle(hFreelanceSymlinkChangeEvent);
+                hFreelanceSymlinkChangeEvent = 0;         
+                return;                            
+            }                                      
             cm_noteLocalMountPointChange();
         }
     }
 }
 #endif
 
+void                                          
+cm_FreelanceShutdown(void)                    
+{                                             
+    freelance_ShutdownFlag = 1;               
+#if !defined(DJGPP)                           
+    if (hFreelanceChangeEvent != 0)           
+        thrd_SetEvent(hFreelanceChangeEvent); 
+    if (hFreelanceSymlinkChangeEvent != 0)           
+        thrd_SetEvent(hFreelanceSymlinkChangeEvent); 
+#endif                                        
+}                                             
+
 void cm_InitFreelance() {
 #if !defined(DJGPP)
     thread_t phandle;
@@ -122,6 +152,10 @@ void cm_InitFreelance() {
 
     lock_InitializeMutex(&cm_Freelance_Lock, "Freelance Lock");
 
+    // make sure we sync the data version to the cached root scache_t                  
+    if (cm_data.rootSCachep && cm_data.rootSCachep->fid.cell == AFS_FAKE_ROOT_CELL_ID) 
+        cm_data.fakeDirVersion = cm_data.rootSCachep->dataVersion;                          
+                                                                                      
     // yj: first we make a call to cm_initLocalMountPoints
     // to read all the local mount points from an ini file
     cm_InitLocalMountPoints();
@@ -183,7 +217,7 @@ void cm_InitFakeRootDir() {
     /* Reserve 2 directory chunks for "." and ".." */
     curChunk += 2;
 
-    while (curDirEntry!=cm_noLocalMountPoints) {
+    while (curDirEntry<cm_noLocalMountPoints) {
         sizeOfCurEntry = cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0);
         if ((curChunk + sizeOfCurEntry >= CPP) ||
              (curDirEntryInPage + 1 >= CM_DIR_EPP)) {
@@ -248,7 +282,7 @@ void cm_InitFakeRootDir() {
     // 2. we have less than CM_DIR_EPP entries in page 0
     // 3. we're not out of chunks in page 0
 
-    while( (curDirEntry!=cm_noLocalMountPoints) && 
+    while( (curDirEntry<cm_noLocalMountPoints) && 
            (curDirEntryInPage < CM_DIR_EPP) &&
            (curChunk + cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0) <= CPP)) 
     {       
@@ -277,7 +311,7 @@ void cm_InitFakeRootDir() {
     curPage++;
 
     // ok, page 0's done. Move on to the next page.
-    while (curDirEntry!=cm_noLocalMountPoints) {
+    while (curDirEntry<cm_noLocalMountPoints) {
         // setup a new page
         curChunk = 1;			// the zeroth chunk is reserved for page header
         curDirEntryInPage = 0; 
@@ -289,7 +323,7 @@ void cm_InitFakeRootDir() {
         fakePageHeader.tag = htons(1234);
 
         // while we're on the same page...
-        while ( (curDirEntry!=cm_noLocalMountPoints) &&
+        while ( (curDirEntry<cm_noLocalMountPoints) &&
                 (curDirEntryInPage < CM_DIR_EPP) &&
                 (curChunk + cm_NameEntries((cm_localMountPoints+curDirEntry)->namep, 0) <= CPP))
         {
@@ -334,7 +368,7 @@ int cm_FakeRootFid(cm_fid_t *fidp)
 /* called while not holding freelance lock */
 int cm_noteLocalMountPointChange(void) {
     lock_ObtainMutex(&cm_Freelance_Lock);
-    cm_fakeDirVersion++;
+    cm_data.fakeDirVersion++;
     cm_localMountPointChangeFlag = 1;
     lock_ReleaseMutex(&cm_Freelance_Lock);
     return 1;
@@ -370,7 +404,7 @@ int cm_reInitLocalMountPoints() {
     lock_ObtainMutex(&cm_Freelance_Lock);  /* always scache then freelance lock */
     for (i=0; i<cm_noLocalMountPoints; i++) {
         hash = CM_SCACHE_HASH(&aFid);
-        for (scp=cm_hashTablep[hash]; scp; scp=scp->nextp) {
+        for (scp=cm_data.hashTablep[hash]; scp; scp=scp->nextp) {
             if (scp->fid.volume == aFid.volume &&
                  scp->fid.vnode == aFid.vnode &&
                  scp->fid.unique == aFid.unique 
@@ -387,12 +421,17 @@ int cm_reInitLocalMountPoints() {
                 cm_ReleaseSCacheNoLock(scp);
 
                 // take the scp out of the hash
-                lscpp = &cm_hashTablep[hash];
-                for (tscp=*lscpp; tscp; lscpp = &tscp->nextp, tscp = *lscpp) {
-                    if (tscp == scp) break;
+                for (lscpp = &cm_data.hashTablep[hash], tscp = cm_data.hashTablep[hash]; 
+                     tscp; 
+                     lscpp = &tscp->nextp, tscp = tscp->nextp) {
+                    if (tscp == scp) {
+                        *lscpp = scp->nextp;
+			lock_ObtainMutex(&scp->mx);
+                        scp->flags &= ~CM_SCACHEFLAG_INHASH;
+			lock_ReleaseMutex(&scp->mx);
+                        break;
+                    }
                 }
-                *lscpp = scp->nextp;
-                scp->flags &= ~CM_SCACHEFLAG_INHASH;
             }
         }
         aFid.vnode = aFid.vnode + 1;
@@ -452,7 +491,7 @@ long cm_InitLocalMountPoints() {
 
 #if !defined(DJGPP)
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
                       &hkFreelance) == ERROR_SUCCESS) {
@@ -477,14 +516,14 @@ long cm_InitLocalMountPoints() {
             rootCellName[0] = '.';
             code = cm_GetRootCellName(&rootCellName[1]);
             if (code == 0) {
-                cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell", 0, NULL);
-                cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell", 1, NULL);
+                cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell.", 0, NULL);
+                cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell.", 1, NULL);
                 dwMountPoints = 2;
             }
         }
 
         if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                          "SOFTWARE\\OpenAFS\\Client\\Freelance\\Symlinks",
+                          AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                           0,
                           NULL,
                           REG_OPTION_NON_VOLATILE,
@@ -542,14 +581,23 @@ long cm_InitLocalMountPoints() {
             if (t2)
                 *(t2+1) = '\0';
 
+            for ( t=line;*t;t++ ) {
+                if ( !isprint(*t) ) {
+                    afsi_log("error occurred while parsing mountpoint entry [%d]: non-printable character", dwIndex);
+                    fprintf(stderr, "error occurred while parsing mountpoint entry [%d]: non-printable character", dwIndex);
+                    cm_noLocalMountPoints--;
+                    continue;
+                }
+            }
+
             // line is not empty, so let's parse it
             t = strchr(line, '#');
             if (!t)
                 t = strchr(line, '%');
             // make sure that there is a '#' or '%' separator in the line
             if (!t) {
-                afsi_log("error occurred while parsing entry in %s: no # or %% separator in line %d", AFS_FREELANCE_INI, dwIndex);
-                fprintf(stderr, "error occurred while parsing entry in afs_freelance.ini: no # or %% separator in line %d", dwIndex);
+                afsi_log("error occurred while parsing mountpoint entry [%d]: no # or %% separator", dwIndex);
+                fprintf(stderr, "error occurred while parsing mountpoint entry [%d]: no # or %% separator", dwIndex);
                 cm_noLocalMountPoints--;
                 continue;
             }
@@ -590,13 +638,22 @@ long cm_InitLocalMountPoints() {
             if (t2)
                 *(t2+1) = '\0';
 
+            for ( t=line;*t;t++ ) {
+                if ( !isprint(*t) ) {
+                    afsi_log("error occurred while parsing symlink entry [%d]: non-printable character", dwIndex);
+                    fprintf(stderr, "error occurred while parsing symlink entry [%d]: non-printable character", dwIndex);
+                    cm_noLocalMountPoints--;
+                    continue;
+                }
+            }
+
             // line is not empty, so let's parse it
             t = strchr(line, ':');
 
             // make sure that there is a ':' separator in the line
             if (!t) {
-                afsi_log("error occurred while parsing symlink entry: no ':' separator in line %d", dwIndex);
-                fprintf(stderr, "error occurred while parsing symlink entry: no ':' separator in line %d", dwIndex);
+                afsi_log("error occurred while parsing symlink entry [%d]: no ':' separator", dwIndex);
+                fprintf(stderr, "error occurred while parsing symlink entry [%d]: no ':' separator", dwIndex);
                 cm_noLocalMountPoints--;
                 continue;
             }
@@ -642,7 +699,7 @@ long cm_InitLocalMountPoints() {
 
 #if !defined(DJGPP)
     RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                    "SOFTWARE\\OpenAFS\\Client\\Freelance",
+                    AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                     0,
                     NULL,
                     REG_OPTION_NON_VOLATILE,
@@ -660,8 +717,8 @@ long cm_InitLocalMountPoints() {
         rootCellName[0] = '.';
       	code = cm_GetRootCellName(&rootCellName[1]);
         if (code == 0) {
-            cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell", 0, NULL);
-            cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell", 1, NULL);
+            cm_FreelanceAddMount(&rootCellName[1], &rootCellName[1], "root.cell.", 0, NULL);
+            cm_FreelanceAddMount(rootCellName, &rootCellName[1], "root.cell.", 1, NULL);
         }
         return 0;
     }
@@ -718,7 +775,7 @@ long cm_InitLocalMountPoints() {
             /* we are migrating to the registry */
             sprintf(szIndex,"%d",dwIndex++);
             dwType = REG_SZ;
-            dwSize = strlen(line) + 1;
+            dwSize = (DWORD)strlen(line) + 1;
             RegSetValueEx( hkFreelance, szIndex, 0, dwType, line, dwSize);
         }
 #endif 
@@ -776,7 +833,7 @@ long cm_FreelanceMountPointExists(char * filename)
     lock_ObtainMutex(&cm_Freelance_Lock);
 
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_QUERY_VALUE,
                       &hkFreelance) == ERROR_SUCCESS) 
@@ -853,7 +910,7 @@ long cm_FreelanceSymlinkExists(char * filename)
     lock_ObtainMutex(&cm_Freelance_Lock);
 
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance\\Symlinks",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                       0,
                       KEY_READ|KEY_QUERY_VALUE,
                       &hkFreelance) == ERROR_SUCCESS) 
@@ -962,7 +1019,7 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
 
 #if !defined(DJGPP)
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
                       &hkFreelance) == ERROR_SUCCESS) {
@@ -998,16 +1055,16 @@ long cm_FreelanceAddMount(char *filename, char *cellname, char *volume, int rw,
             if (RegQueryValueEx( hkFreelance, szIndex, 0, &dwType, szMount, &dwSize) != ERROR_SUCCESS) {
                 /* found an unused value */
                 dwType = REG_SZ;
-                dwSize = strlen(line) + 1;
+                dwSize = (DWORD)strlen(line) + 1;
                 RegSetValueEx( hkFreelance, szIndex, 0, dwType, line, dwSize);
                 break;
             } else {
-                int len = strlen(filename);
+                int len = (int)strlen(filename);
                 if ( dwType == REG_SZ && !strncmp(filename, szMount, len) && 
                      (szMount[len] == '%' || szMount[len] == '#')) {
                     /* Replace the existing value */
                     dwType = REG_SZ;
-                    dwSize = strlen(line) + 1;
+                    dwSize = (DWORD)strlen(line) + 1;
                     RegSetValueEx( hkFreelance, szIndex, 0, dwType, line, dwSize);
                     break;
                 }
@@ -1065,7 +1122,7 @@ long cm_FreelanceRemoveMount(char *toremove)
 
 #if !defined(DJGPP)
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
                       &hkFreelance) == ERROR_SUCCESS) {
@@ -1196,7 +1253,7 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
 
 #if !defined(DJGPP)
     if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                        "SOFTWARE\\OpenAFS\\Client\\Freelance\\Symlinks",
+                        AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                         0,
                         NULL,
                         REG_OPTION_NON_VOLATILE,
@@ -1233,15 +1290,15 @@ long cm_FreelanceAddSymlink(char *filename, char *destination, cm_fid_t *fidp)
             if (RegQueryValueEx( hkFreelanceSymlinks, szIndex, 0, &dwType, szLink, &dwSize) != ERROR_SUCCESS) {
                 /* found an unused value */
                 dwType = REG_SZ;
-                dwSize = strlen(line) + 1;
+                dwSize = (DWORD)strlen(line) + 1;
                 RegSetValueEx( hkFreelanceSymlinks, szIndex, 0, dwType, line, dwSize);
                 break;
             } else {
-                int len = strlen(filename);
+                int len = (int)strlen(filename);
                 if ( dwType == REG_SZ && !strncmp(filename, szLink, len) && szLink[len] == ':') {
                     /* Replace the existing value */
                     dwType = REG_SZ;
-                    dwSize = strlen(line) + 1;
+                    dwSize = (DWORD)strlen(line) + 1;
                     RegSetValueEx( hkFreelanceSymlinks, szIndex, 0, dwType, line, dwSize);
                     break;
                 }
@@ -1278,7 +1335,7 @@ long cm_FreelanceRemoveSymlink(char *toremove)
 
 #if !defined(DJGPP)
     if (RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                      "SOFTWARE\\OpenAFS\\Client\\Freelance\\Symlinks",
+                      AFSREG_CLT_OPENAFS_SUBKEY "\\Freelance\\Symlinks",
                       0,
                       KEY_READ|KEY_WRITE|KEY_QUERY_VALUE,
                       &hkFreelanceSymlinks) == ERROR_SUCCESS) {
