@@ -41,6 +41,9 @@
 
 #include "cm_rpc.h"
 #include <strsafe.h>
+#include <winioctl.h>
+#include <..\afsrdr\kif.h>
+#include <rx\rx.h>
 
 #ifdef _DEBUG
 #include <crtdbg.h>
@@ -62,7 +65,6 @@ void cm_InitIoctl(void)
     lock_InitializeMutex(&cm_Afsdsbmt_Lock, "AFSDSBMT.INI Access Lock");
 }
 
-
 long cm_CleanFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 {
     long code;
@@ -83,13 +85,6 @@ long cm_FlushFile(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 {
     long code;
 
-#ifdef AFS_FREELANCE_CLIENT
-    if ( scp->fid.cell == AFS_FAKE_ROOT_CELL_ID && scp->fid.volume == AFS_FAKE_ROOT_VOL_ID ) {
-	cm_noteLocalMountPointChange();
-	return 0;
-    }
-#endif
-
     lock_ObtainWrite(&scp->bufCreateLock);
     code = buf_FlushCleanPages(scp, userp, reqp);
         
@@ -107,12 +102,12 @@ long cm_FlushParent(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 {
     long code = 0;
     cm_scache_t * pscp;
-
+  
     pscp = cm_FindSCacheParent(scp);
-
+  
     /* now flush the file */
     code = cm_FlushFile(pscp, userp, reqp);
-    cm_ReleaseSCache(pscp);
+    cm_ReleaseSCache(scp);
 
     return code;
 }
@@ -124,13 +119,6 @@ long cm_FlushVolume(cm_user_t *userp, cm_req_t *reqp, afs_uint32 cell, afs_uint3
     cm_scache_t *scp;
     int i;
 
-#ifdef AFS_FREELANCE_CLIENT
-    if ( cell == AFS_FAKE_ROOT_CELL_ID && volume == AFS_FAKE_ROOT_VOL_ID ) {
-	cm_noteLocalMountPointChange();
-	return 0;
-    }
-#endif
-
     lock_ObtainWrite(&cm_scacheLock);
     for (i=0; i<cm_data.hashTableSize; i++) {
         for (scp = cm_data.hashTablep[i]; scp; scp = scp->nextp) {
@@ -213,7 +201,12 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
     long code;
     cm_scache_t *substRootp;
     char * relativePath = ioctlp->inDatap;
-
+#ifdef AFSIFS
+    char * absRoot[MAX_PATH];
+    long length;
+    wchar_t absRoot_w[MAX_PATH];
+    HANDLE rootDir;
+#endif
     osi_Log1(afsd_logp, "cm_ParseIoctlPath %s", osi_LogSaveString(afsd_logp,relativePath));
 
     /* This is usually the file name, but for StatMountPoint it is the path. */
@@ -225,6 +218,28 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
      */
     TranslateExtendedChars(relativePath);
 
+#ifdef AFSIFS
+    /* we have passed the whole path, including the afs prefix.
+	   when the pioctl call is made, we perform an ioctl to afsrdr
+	   and it returns the correct (full) path.  therefore, there is
+	   no drive letter, and the path is absolute. */
+    code = cm_NameI(cm_data.rootSCachep, relativePath,
+                     CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
+                     userp, "", reqp, scpp);
+
+    if (code)
+        return code;
+
+    /* # of bytes of path */
+    code = strlen(ioctlp->inDatap) + 1;
+    ioctlp->inDatap += code;
+
+    /* This is usually nothing, but for StatMountPoint it is the file name. */
+    TranslateExtendedChars(ioctlp->inDatap);
+
+    return 0;
+#endif /* AFSIFS */
+
     if (relativePath[0] == relativePath[1] &&
          relativePath[1] == '\\' && 
          !_strnicmp(cm_NetbiosName,relativePath+2,strlen(cm_NetbiosName))) 
@@ -239,7 +254,7 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
          * since it had better expand into the value of ioctl->tidPathp
          */
         char * p;
-        p = relativePath + 2 + strlen(cm_NetbiosName) + 1;
+        p = relativePath + 2 + strlen(cm_NetbiosName) + 1;			/* buffer overflow vuln.? */
         if ( !_strnicmp("all", p, 3) )
             p += 4;
 
@@ -256,17 +271,13 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
                              CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, sharePath, reqp, &substRootp);
             free(sharePath);
-            if (code) {
-		osi_Log1(afsd_logp,"cm_ParseIoctlPath [1] code 0x%x", code);
+            if (code) 
                 return code;
-	    }
 
             code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, NULL, reqp, scpp);
-            if (code) {
-		osi_Log1(afsd_logp,"cm_ParseIoctlPath [2] code 0x%x", code);
+            if (code) 
                 return code;
-	    }
         } else {
             /* otherwise, treat the name as a cellname mounted off the afs root.
              * This requires that we reconstruct the shareName string with 
@@ -288,49 +299,35 @@ long cm_ParseIoctlPath(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
             code = cm_NameI(cm_data.rootSCachep, ioctlp->prefix->data,
                              CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                              userp, shareName, reqp, &substRootp);
-            if (code) {
-		osi_Log1(afsd_logp,"cm_ParseIoctlPath [3] code 0x%x", code);
+            if (code) 
                 return code;
-	    }
 
             code = cm_NameI(substRootp, p, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                             userp, NULL, reqp, scpp);
-            if (code) {
-		cm_ReleaseSCache(substRootp);
-		osi_Log1(afsd_logp,"cm_ParseIoctlPath code [4] 0x%x", code);
+            if (code) 
                 return code;
-	    }
         }
     } else {
         code = cm_NameI(cm_data.rootSCachep, ioctlp->prefix->data,
                          CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                          userp, ioctlp->tidPathp, reqp, &substRootp);
-        if (code) {
-	    osi_Log1(afsd_logp,"cm_ParseIoctlPath [6] code 0x%x", code);
+        if (code) 
             return code;
-	}
         
         code = cm_NameI(substRootp, relativePath, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                          userp, NULL, reqp, scpp);
-        if (code) {
-	    cm_ReleaseSCache(substRootp);
-	    osi_Log1(afsd_logp,"cm_ParseIoctlPath [7] code 0x%x", code);
+        if (code) 
             return code;
-	}
     }
 
     /* # of bytes of path */
-    code = strlen(ioctlp->inDatap) + 1;
+    code = (long)strlen(ioctlp->inDatap) + 1;
     ioctlp->inDatap += code;
 
     /* This is usually nothing, but for StatMountPoint it is the file name. */
     TranslateExtendedChars(ioctlp->inDatap);
 
-    if (substRootp)
-	cm_ReleaseSCache(substRootp);
-
     /* and return success */
-    osi_Log1(afsd_logp,"cm_ParseIoctlPath [8] code 0x%x", code);
     return 0;
 }
 
@@ -338,7 +335,7 @@ void cm_SkipIoctlPath(smb_ioctl_t *ioctlp)
 {
     long temp;
         
-    temp = strlen(ioctlp->inDatap) + 1;
+    temp = (long) strlen(ioctlp->inDatap) + 1;
     ioctlp->inDatap += temp;
 }       
 
@@ -479,13 +476,13 @@ long cm_ParseIoctlParent(smb_ioctl_t *ioctlp, cm_user_t *userp, cm_req_t *reqp,
                         userp, ioctlp->tidPathp, reqp, &substRootp);
         if (code) return code;
 
-        code = cm_NameI(substRootp, tbuffer, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
+        code = cm_NameI(substRootp, tbuffer, CM_FLAG_FOLLOW,
                         userp, NULL, reqp, scpp);
         if (code) return code;
     }
 
     /* # of bytes of path */
-    code = strlen(ioctlp->inDatap) + 1;
+    code = (long)strlen(ioctlp->inDatap) + 1;
     ioctlp->inDatap += code;
 
     /* and return success */
@@ -531,7 +528,7 @@ long cm_IoctlGetACL(smb_ioctl_t *ioctlp, cm_user_t *userp)
     if (code) return code;
 
     /* skip over return data */
-    tlen = strlen(ioctlp->outDatap) + 1;
+    tlen = (int)strlen(ioctlp->outDatap) + 1;
     ioctlp->outDatap += tlen;
 
     /* and return success */
@@ -598,7 +595,7 @@ long cm_IoctlSetACL(struct smb_ioctl *ioctlp, struct cm_user *userp)
     fid.Unique = scp->fid.unique;
     do {
         acl.AFSOpaque_val = ioctlp->inDatap;
-        acl.AFSOpaque_len = strlen(ioctlp->inDatap)+1;
+        acl.AFSOpaque_len = (u_int)strlen(ioctlp->inDatap)+1;
         code = cm_Conn(&scp->fid, userp, &req, &connp);
         if (code) continue;
 
@@ -1129,7 +1126,7 @@ long cm_IoctlCheckVolumes(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
 long cm_IoctlSetCacheSize(struct smb_ioctl *ioctlp, struct cm_user *userp)
 {
-    long temp;
+    afs_uint64 temp;
     long code;
 
     cm_SkipIoctlPath(ioctlp);
@@ -1167,10 +1164,12 @@ long cm_IoctlTraceControl(struct smb_ioctl *ioctlp, struct cm_user *userp)
         if ((inValue & 1) == 0) {
             /* disable tracing */
             osi_LogDisable(afsd_logp);
+	    rx_DebugOnOff(FALSE);
         }
         else {
             /* enable tracing */
             osi_LogEnable(afsd_logp);
+	    rx_DebugOnOff(TRUE);
         }
     }
 
@@ -1365,7 +1364,7 @@ long cm_IoctlSysName(struct smb_ioctl *ioctlp, struct cm_user *userp)
         cp2 = ioctlp->inDatap;
         for ( cp=ioctlp->inDatap, count = 0; count < setSysName; count++ ) {
             /* won't go past end of ioctlp->inDatap since maxsysname*num < ioctlp->inDatap length */
-            t = strlen(cp);
+            t = (int)strlen(cp);
             if (t >= MAXSYSNAME || t <= 0)
                 return EINVAL;
             /* check for names that can shoot us in the foot */
@@ -1377,7 +1376,7 @@ long cm_IoctlSysName(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
         /* inname gets first entry in case we're being a translator */
         /* (we are never a translator) */
-        t = strlen(ioctlp->inDatap);
+        t = (int)strlen(ioctlp->inDatap);
         memcpy(inname, ioctlp->inDatap, t + 1);
         ioctlp->inDatap += t + 1;
         num = count;
@@ -1404,7 +1403,7 @@ long cm_IoctlSysName(struct smb_ioctl *ioctlp, struct cm_user *userp)
                 if (!cm_sysNameList[count])
                     osi_panic("cm_IoctlSysName: no cm_sysNameList entry to write\n",
                                __FILE__, __LINE__);
-                t = strlen(cp);
+                t = (int)strlen(cp);
                 StringCbCopyA(cm_sysNameList[count], MAXSYSNAME, cp);
                 cp += t + 1;
             }
@@ -1424,7 +1423,7 @@ long cm_IoctlSysName(struct smb_ioctl *ioctlp, struct cm_user *userp)
                 if ( !(*sysnamelist)[count] )
                     osi_panic("cm_IoctlSysName: no cm_sysNameList entry to read\n", 
                                __FILE__, __LINE__);
-                t = strlen((*sysnamelist)[count]);
+                t = (int)strlen((*sysnamelist)[count]);
                 if (t >= MAXSYSNAME)
                     osi_panic("cm_IoctlSysName: sysname entry garbled\n", 
                                __FILE__, __LINE__);
@@ -1954,6 +1953,7 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
         uname = tp;
         tp += strlen(tp) + 1;
 
+#ifndef AFSIFS			/* no SMB username, so we cannot log based on this */
         if (flags & PIOCTL_LOGON) {
             /* SMB user name with which to associate tokens */
             smbname = tp;
@@ -1965,6 +1965,7 @@ long cm_IoctlSetToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
             osi_Log1(smb_logp,"cm_IoctlSetToken for user [%s]",
                      osi_LogSaveString(smb_logp,uname));
         }
+#endif
 
 #ifndef DJGPP   /* for win95, session key is back in pioctl */
 		/* uuid */
@@ -2232,11 +2233,6 @@ long cm_IoctlDelToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
         free(ucellp->ticketp);
         ucellp->ticketp = NULL;
     }
-    ucellp->ticketLen = 0;
-    memset(ucellp->sessionKey.data, 0, 8);
-    ucellp->kvno = 0;
-    ucellp->expirationTime = 0;
-    ucellp->userName[0] = '\0';
     ucellp->flags &= ~CM_UCELLFLAG_RXKAD;
     ucellp->gen++;
 
@@ -2255,16 +2251,6 @@ long cm_IoctlDelAllToken(struct smb_ioctl *ioctlp, struct cm_user *userp)
 
     for (ucellp = userp->cellInfop; ucellp; ucellp = ucellp->nextp) {
         osi_Log1(smb_logp,"cm_IoctlDelAllToken ucellp %lx", ucellp);
-
-	if (ucellp->ticketp) {
-	    free(ucellp->ticketp);
-	    ucellp->ticketp = NULL;
-	}
-	ucellp->ticketLen = 0;
-	memset(ucellp->sessionKey.data, 0, 8);
-	ucellp->kvno = 0;
-	ucellp->expirationTime = 0;
-	ucellp->userName[0] = '\0';
         ucellp->flags &= ~CM_UCELLFLAG_RXKAD;
         ucellp->gen++;
     }
@@ -2335,7 +2321,7 @@ long cm_IoctlMakeSubmount(smb_ioctl_t *ioctlp, cm_user_t *userp)
                            (strlen(&afspath[strlen(cm_mountRoot)])) ?
                            &afspath[strlen(cm_mountRoot)]:"/",
                            (strlen(&afspath[strlen(cm_mountRoot)])) ?
-                           strlen(&afspath[strlen(cm_mountRoot)])+1:2);
+                           (DWORD)strlen(&afspath[strlen(cm_mountRoot)])+1:2);
 
             RegCloseKey( hkSubmounts );
             StringCbCopyA(ioctlp->outDatap, 999999, submountreqp);
@@ -2443,7 +2429,7 @@ long cm_IoctlMakeSubmount(smb_ioctl_t *ioctlp, cm_user_t *userp)
                    (strlen(&afspath[strlen(cm_mountRoot)])) ?
                    &afspath[strlen(cm_mountRoot)]:"/",
                    (strlen(&afspath[strlen(cm_mountRoot)])) ?
-                   strlen(&afspath[strlen(cm_mountRoot)])+1:2);
+                   (DWORD)strlen(&afspath[strlen(cm_mountRoot)])+1:2);
 
     ioctlp->outDatap += strlen(ioctlp->outDatap) +1;
     RegCloseKey(hkSubmounts);
@@ -2461,10 +2447,18 @@ long cm_IoctlGetRxkcrypt(smb_ioctl_t *ioctlp, cm_user_t *userp)
 
 long cm_IoctlSetRxkcrypt(smb_ioctl_t *ioctlp, cm_user_t *userp)
 {
+    afs_int32 c = cryptall;
+
     cm_SkipIoctlPath(ioctlp);
 
     memcpy(&cryptall, ioctlp->inDatap, sizeof(cryptall));
 
+    if (c != cryptall) {
+	if (cryptall)
+            LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_ON);
+	else
+            LogEvent(EVENTLOG_INFORMATION_TYPE, MSG_CRYPT_OFF);
+    }
     return 0;
 }
 
