@@ -17,7 +17,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_create.c,v 1.16.2.3 2004/11/09 17:15:04 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_create.c,v 1.23 2005/10/15 14:23:21 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -30,10 +30,6 @@ RCSID
  * I think we can get away without it, but I'm not sure.  Note that
  * afs_setattr is called in here for truncation.
  */
-#ifdef	AFS_OSF_ENV
-int
-afs_create(struct nameidata *ndp, struct vattr *attrs)
-#else /* AFS_OSF_ENV */
 #ifdef AFS_SGI64_ENV
 int
 afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs, int flags,
@@ -44,17 +40,7 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	   enum vcexcl aexcl, int amode, struct vcache **avcp,
 	   struct AFS_UCRED *acred)
 #endif				/* AFS_SGI64_ENV */
-#endif				/* AFS_OSF_ENV */
 {
-#ifdef AFS_OSF_ENV
-    register struct vcache *adp = VTOAFS(ndp->ni_dvp);
-    char *aname = ndp->ni_dent.d_name;
-    enum vcexcl aexcl = NONEXCL;	/* XXX - create called properly */
-    int amode = 0;		/* XXX - checked in higher level */
-    struct vcache **avcp = (struct vcache **)&(ndp->ni_vp);
-    struct ucred *acred = ndp->ni_cred;
-#endif
-
     afs_int32 origCBs, origZaps, finalZaps;
     struct vrequest treq;
     register afs_int32 code;
@@ -193,7 +179,9 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 		    code = EACCES;
 		    goto done;
 		}
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#if defined(AFS_DARWIN80_ENV)
+		if ((amode & VWRITE) || VATTR_IS_ACTIVE(attrs, va_data_size))
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 		if ((amode & VWRITE) || (attrs->va_mask & AT_SIZE))
 #else
 		if ((amode & VWRITE) || len != 0xffffffff)
@@ -210,7 +198,9 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 			goto done;
 		    }
 		}
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#if defined(AFS_DARWIN80_ENV)
+		if (VATTR_IS_ACTIVE(attrs, va_data_size))
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 		if (attrs->va_mask & AT_SIZE)
 #else
 		if (len != 0xffffffff)
@@ -222,7 +212,11 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 			goto done;
 		    }
 		    /* do a truncate */
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#if defined(AFS_DARWIN80_ENV)
+		    VATTR_INIT(attrs);
+		    VATTR_SET_SUPPORTED(attrs, va_data_size);
+		    VATTR_SET_ACTIVE(attrs, va_data_size);
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 		    attrs->va_mask = AT_SIZE;
 #else
 		    VATTR_NULL(attrs);
@@ -311,43 +305,33 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	     (tc, code, &adp->fid, &treq, AFS_STATS_FS_RPCIDX_CREATEFILE,
 	      SHARED_LOCK, NULL));
 
-#if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV)
-    if (code == EEXIST && aexcl == NONEXCL) {
-	/* This lookup was handled in the common vn_open code in the
-	 * vnode layer */
-	if (tdc) {
-	    ReleaseSharedLock(&tdc->lock);
-	    afs_PutDCache(tdc);
-	}
-	ReleaseWriteLock(&adp->lock);
-	goto done;
-    }
-#else /* AFS_OSF_ENV */
+    if (code == EEXIST &&
 #ifdef AFS_SGI64_ENV
-    if (code == EEXIST && !(flags & VEXCL)) {
+    !(flags & VEXCL)
 #else /* AFS_SGI64_ENV */
-    if (code == EEXIST && aexcl == NONEXCL) {
-#endif /* AFS_SGI64_ENV */
+    aexcl == NONEXCL
+#endif
+    ) {
 	/* if we get an EEXIST in nonexcl mode, just do a lookup */
 	if (tdc) {
 	    ReleaseSharedLock(&tdc->lock);
 	    afs_PutDCache(tdc);
 	}
 	ReleaseWriteLock(&adp->lock);
+#if !(defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV))
 #if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 #if defined(AFS_SGI64_ENV)
-	code =
-	    afs_lookup(VNODE_TO_FIRST_BHV((vnode_t *) adp), aname, avcp, NULL,
-		       0, NULL, acred);
+	code = afs_lookup(VNODE_TO_FIRST_BHV((vnode_t *) adp), aname, avcp, 
+			  NULL, 0, NULL, acred);
 #else
 	code = afs_lookup(adp, aname, avcp, NULL, 0, NULL, acred);
 #endif /* AFS_SGI64_ENV */
 #else /* SUN5 || SGI */
 	code = afs_lookup(adp, aname, avcp, acred);
 #endif /* SUN5 || SGI */
+#endif /* !(AFS_OSF_ENV || AFS_DARWIN_ENV) */
 	goto done;
     }
-#endif /* AFS_OSF_ENV */
     if (code) {
 	if (code < 0) {
 	    ObtainWriteLock(&afs_xcbhash, 488);
@@ -438,6 +422,8 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    }
 	    ReleaseWriteLock(&afs_xcbhash);
 	    afs_ProcessFS(tvc, &OutFidStatus, &treq);
+	    tvc->parentVnode = adp->fid.Fid.Vnode;
+	    tvc->parentUnique = adp->fid.Fid.Unique;
 	    ReleaseWriteLock(&tvc->lock);
 	    *avcp = tvc;
 	    code = 0;
@@ -475,10 +461,6 @@ afs_create(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     code = afs_CheckCode(code, &treq, 20);
 
   done2:
-#ifdef	AFS_OSF_ENV
-    afs_PutVCache(adp);
-#endif /* AFS_OSF_ENV */
-
     return code;
 }
 
@@ -518,9 +500,9 @@ afs_LocalHero(register struct vcache *avc, register struct dcache *adc,
 	hset(avc->m.DataVersion, avers);
 #ifdef AFS_64BIT_CLIENT
 	FillInt64(avc->m.Length, astat->Length_hi, astat->Length);
-#else /* AFS_64BIT_ENV */
+#else /* AFS_64BIT_CLIENT */
 	avc->m.Length = astat->Length;
-#endif /* AFS_64BIT_ENV */
+#endif /* AFS_64BIT_CLIENT */
 	avc->m.Date = astat->ClientModTime;
     }
     if (ok) {
