@@ -21,7 +21,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.31.2.6 2005/01/31 03:49:15 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.48 2006/01/10 15:09:55 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -126,8 +126,6 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
 	      SHARED_LOCK, NULL));
 
     osi_dnlc_remove(adp, aname, tvc);
-    if (tvc)
-	afs_symhint_inval(tvc);	/* XXX: don't really need to be so extreme */
 
     if (code) {
 	if (tdc) {
@@ -169,7 +167,7 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
      * call FindVCache instead of GetVCache since if the file's really
      * gone, we won't be able to fetch the status info anyway.  */
     if (tvc) {
-#if	defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
 	afs_BozonLock(&tvc->pvnLock, tvc);
 	/* Since afs_TryToSmush will do a pvn_vptrunc */
 #endif
@@ -183,7 +181,7 @@ afsremove(register struct vcache *adp, register struct dcache *tdc,
 		afs_TryToSmush(tvc, acred, 0);
 	}
 	ReleaseWriteLock(&tvc->lock);
-#if	defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
 	afs_BozonUnlock(&tvc->pvnLock, tvc);
 #endif
 	afs_PutVCache(tvc);
@@ -358,33 +356,35 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 	FetchWholeEnchilada(tvc, &treq);
 #endif
 	ReleaseWriteLock(&tvc->lock);
+	if (tdc) 
+	    ReleaseSharedLock(&tdc->lock);
 	ObtainWriteLock(&adp->lock, 144);
+	/* Technically I don't think we need this back, but let's hold it 
+	   anyway; The "got" reference should actually be sufficient. */
+	if (tdc) 
+	    ObtainSharedLock(&tdc->lock, 640);
     }
 
     osi_dnlc_remove(adp, aname, tvc);
-    if (tvc)
-	afs_symhint_inval(tvc);
 
     Tadp1 = adp;
+#ifndef AFS_DARWIN80_ENV
     Tadpr = VREFCOUNT(adp);
+#endif
     Ttvc = tvc;
     Tnam = aname;
     Tnam1 = 0;
     if (tvc)
+#ifndef AFS_DARWIN80_ENV
 	Ttvcr = VREFCOUNT(tvc);
+#endif
 #ifdef	AFS_AIX_ENV
-    if (tvc && (VREFCOUNT(tvc) > 2) && tvc->opens > 0
-	&& !(tvc->states & CUnlinked))
+    if (tvc && VREFCOUNT_GT(tvc, 2) && tvc->opens > 0
+	&& !(tvc->states & CUnlinked)) {
 #else
-#ifdef AFS_DARWIN14_ENV
-    if (tvc && (VREFCOUNT(tvc) > 1 + DARWIN_REFBASE) && tvc->opens > 0
-	&& !(tvc->states & CUnlinked))
-#else
-    if (tvc && (VREFCOUNT(tvc) > 1) && tvc->opens > 0
-	&& !(tvc->states & CUnlinked))
+    if (tvc && VREFCOUNT_GT(tvc, 1) && tvc->opens > 0
+	&& !(tvc->states & CUnlinked)) {
 #endif
-#endif
-    {
 	char *unlname = afs_newname();
 
 	ReleaseWriteLock(&adp->lock);
@@ -393,7 +393,12 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 	code = afsrename(adp, aname, adp, unlname, acred, &treq);
 	Tnam1 = unlname;
 	if (!code) {
+	    char *oldmvid = NULL;
+	    if (tvc->mvid) 
+		oldmvid = tvc->mvid;
 	    tvc->mvid = (struct VenusFid *)unlname;
+	    if (oldmvid)
+		osi_FreeSmallSpace(oldmvid);
 	    crhold(acred);
 	    if (tvc->uncred) {
 		crfree(tvc->uncred);
@@ -410,6 +415,7 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 	code = afsremove(adp, tdc, tvc, aname, acred, &treq);
     }
     afs_PutFakeStat(&fakestate);
+    osi_Assert(!WriteLocked(&adp->lock) || (adp->lock.pid_writer != MyPidxx));
     return code;
 }
 
@@ -429,9 +435,6 @@ afs_remunlink(register struct vcache *avc, register int doit)
     struct VenusFid dirFid;
     register struct dcache *tdc;
     afs_int32 code = 0;
-#ifdef AFS_DARWIN14_ENV
-    int oldref;
-#endif
 
     if (NBObtainWriteLock(&avc->lock, 423))
 	return 0;
@@ -448,30 +451,17 @@ afs_remunlink(register struct vcache *avc, register int doit)
 	    cred = avc->uncred;
 	    avc->uncred = NULL;
 
-#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN14_ENV)
-	    /* this is called by vrele (via VOP_INACTIVE) when the refcount
-	     * is 0. we can't just call VN_HOLD since vref will panic.
-	     * we can't just call osi_vnhold because a later AFS_RELE will call
-	     * vrele again, which will try to call VOP_INACTIVE again after
-	     * vn_locking the vnode. which would be fine except that our vrele
-	     * caller also locked the vnode... So instead, we just gimmick the
-	     * refcounts and hope nobody else can touch the file now */
-	    osi_Assert(VREFCOUNT(avc) == 0);
-	    VREFCOUNT_SET(avc, 1);
-#endif
+#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN80_ENV)
+	    VREF(AFSTOV(avc));
+#else
 	    VN_HOLD(AFSTOV(avc));
+#endif
 
 	    /* We'll only try this once. If it fails, just release the vnode.
 	     * Clear after doing hold so that NewVCache doesn't find us yet.
 	     */
 	    avc->states &= ~(CUnlinked | CUnlinkedDel);
 
-#ifdef AFS_DARWIN14_ENV
-	    if (VREFCOUNT(avc) < 4) {
-		oldref = 4 - VREFCOUNT(avc);
-		VREFCOUNT_SET(avc, 4);
-	    }
-#endif
 	    ReleaseWriteLock(&avc->lock);
 
 	    dirFid.Cell = avc->fid.Cell;
@@ -495,17 +485,6 @@ afs_remunlink(register struct vcache *avc, register int doit)
 	    }
 	    osi_FreeSmallSpace(unlname);
 	    crfree(cred);
-#ifdef AFS_DARWIN_ENV
-#ifndef AFS_DARWIN14_ENV
-	    osi_Assert(VREFCOUNT(avc) == 1);
-	    VREFCOUNT_SET(avc, 0);
-#else
-	    if (oldref) {
-		int newref = VREFCOUNT(avc) - oldref;
-		VREFCOUNT_SET(avc, newref);
-	    }
-#endif
-#endif
 	}
     } else {
 	ReleaseWriteLock(&avc->lock);
