@@ -10,6 +10,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.41 2006/02/13 18:29:26 shadow Exp $");
 
 #ifdef AFS_AIX51_ENV
 #define __FULL_PROTO
@@ -38,8 +40,6 @@ static int rxepoch_checked = 0;
 #define afs_CheckRXEpoch() {if (rxepoch_checked == 0 && rxkad_EpochWasSet) { \
 	rxepoch_checked = 1; afs_GCUserData(/* force flag */ 1);  } }
 
-extern int afsd_dynamic_vcaches;
-
 /* PAG garbage collection */
 /* We induce a compile error if param.h does not define AFS_GCPAGS */
 afs_int32 afs_gcpags = AFS_GCPAGS;
@@ -49,7 +49,7 @@ afs_int32 afs_CheckServerDaemonStarted = 0;
 #ifdef DEFAULT_PROBE_INTERVAL
 afs_int32 PROBE_INTERVAL = DEFAULT_PROBE_INTERVAL;	/* overridding during compile */
 #else
-afs_int32 PROBE_INTERVAL = 30;	/* default to 3 min */
+afs_int32 PROBE_INTERVAL = 180;	/* default to 3 min */
 #endif
 
 #define PROBE_WAIT() (1000 * (PROBE_INTERVAL - ((afs_random() & 0x7fffffff) \
@@ -109,11 +109,6 @@ extern int vfs_context_ref;
 #else
 #define vfs_context_ref 1
 #endif
-
-/* This function always holds the GLOCK whilst it is running. The caller
- * gets the GLOCK before invoking it, and afs_osi_Sleep drops the GLOCK
- * whilst we are sleeping, and regains it when we're woken up.
- */
 void
 afs_Daemon(void)
 {
@@ -121,13 +116,13 @@ afs_Daemon(void)
     struct afs_exporter *exporter;
     afs_int32 now;
     afs_int32 last3MinCheck, last10MinCheck, last60MinCheck, lastNMinCheck;
-    afs_int32 last1MinCheck, last5MinCheck;
+    afs_int32 last1MinCheck;
     afs_uint32 lastCBSlotBump;
     char cs_warned = 0;
 
     AFS_STATCNT(afs_Daemon);
     last1MinCheck = last3MinCheck = last60MinCheck = last10MinCheck =
-    last5MinCheck = lastNMinCheck = 0;
+	lastNMinCheck = 0;
 
     afs_rootFid.Fid.Volume = 0;
     while (afs_initState < 101)
@@ -155,7 +150,6 @@ afs_Daemon(void)
     last3MinCheck = now - 90 + ((afs_random() & 0x7fffffff) % 180);
     last60MinCheck = now - 1800 + ((afs_random() & 0x7fffffff) % 3600);
     last10MinCheck = now - 300 + ((afs_random() & 0x7fffffff) % 600);
-    last5MinCheck = now - 150 + ((afs_random() & 0x7fffffff) % 300);
     lastNMinCheck = now - 90 + ((afs_random() & 0x7fffffff) % 180);
 
     /* start off with afs_initState >= 101 (basic init done) */
@@ -197,22 +191,6 @@ afs_Daemon(void)
 					 * tickets */
 	    last3MinCheck = now;
 	}
-#ifdef AFS_MAXVCOUNT_ENV
-    if (afsd_dynamic_vcaches && (last5MinCheck + 300 < now)) {
-        /* start with trying to drop us back to our base usage */
-        int anumber;
-        if (afs_maxvcount <= afs_cacheStats) 
-        anumber = VCACHE_FREE;
-        else
-        anumber = VCACHE_FREE + (afs_maxvcount - afs_cacheStats);
-
-	ObtainWriteLock(&afs_xvcache, 734);
-	afs_ShakeLooseVCaches(anumber);
-	ReleaseWriteLock(&afs_xvcache);
-	last5MinCheck = now;
-    }
-#endif
-
 	if (!afs_CheckServerDaemonStarted) {
 	    /* Do the check here if the correct afsd is not installed. */
 	    if (!cs_warned) {
@@ -376,11 +354,7 @@ afs_CheckRootVolume(void)
 		    dp = d_find_alias(AFSTOV(afs_globalVp));
 		    
 #if defined(AFS_LINUX24_ENV)
-#if defined(HAVE_DCACHE_LOCK)
 		    spin_lock(&dcache_lock);
-#else
-		    spin_lock(&AFSTOV(vcp)->i_lock);
-#endif
 #if defined(AFS_LINUX26_ENV)
 		    spin_lock(&dp->d_lock);
 #endif
@@ -392,11 +366,7 @@ afs_CheckRootVolume(void)
 #if defined(AFS_LINUX26_ENV)
 		    spin_unlock(&dp->d_lock);
 #endif
-#if defined(HAVE_DCACHE_LOCK)
 		    spin_unlock(&dcache_lock);
-#else
-		    spin_unlock(&AFSTOV(vcp)->i_lock);
-#endif
 #endif
 		    dput(dp);
 		    
