@@ -31,11 +31,7 @@
   *	-confdir    The configuration directory .
   *	-nosettime  Don't keep checking the time to avoid drift (default).
   *     -settime    Keep checking the time to avoid drift.
-  *	-rxmaxmtu   Set the max mtu to help with VPN issues.
   *	-verbose     Be chatty.
-  *	-disable-dynamic-vcaches     Disable the use of -stat value as the starting size of
-  *                          the size of the vcache/stat cache pool, 
-  *                          but increase that pool dynamically as needed.
   *	-debug	   Print out additional debugging info.
   *	-kerndev    [OBSOLETE] The kernel device for AFS.
   *	-dontfork   [OBSOLETE] Don't fork off as a new process.
@@ -61,6 +57,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/afsd/afsd.c,v 1.59 2005/12/01 04:02:23 shadow Exp $");
 
 #define VFS 1
 
@@ -162,42 +160,11 @@ void set_staticaddrs(void);
 #include <sys/ioctl.h>
 #endif
 #include <mach/mach.h>
-#ifndef AFS_DARWIN100_ENV
 /* Symbols from the DiskArbitration framework */
 kern_return_t DiskArbStart(mach_port_t *);
 kern_return_t DiskArbDiskAppearedWithMountpointPing_auto(char *, unsigned int,
 							 char *);
 #define DISK_ARB_NETWORK_DISK_FLAG 8
-#endif
-#include <mach/mach_port.h>
-#include <mach/mach_interface.h>
-#include <mach/mach_init.h>
-
-#include <CoreFoundation/CoreFoundation.h>
-
-#include <SystemConfiguration/SystemConfiguration.h>
-#include <SystemConfiguration/SCDynamicStore.h>
-
-#include <IOKit/pwr_mgt/IOPMLib.h>
-#include <IOKit/IOMessage.h>
-
-#include <dns_sd.h>
-
-typedef struct DNSSDState
-{
-    DNSServiceRef       service;
-    CFRunLoopSourceRef  source;
-    CFSocketRef         socket;
-} DNSSDState;
-
-static io_connect_t root_port;
-static IONotificationPortRef notify;
-static io_object_t iterator;
-static CFRunLoopSourceRef source;
-static DNSSDState dnsstate;
-
-static int event_pid;
-
 #endif /* AFS_DARWIN_ENV */
 
 #ifndef MOUNT_AFS
@@ -266,7 +233,9 @@ struct in_addr_42 {
 afs_int32 enable_rxbind = 0;
 afs_int32 afs_shutdown = 0;
 afs_int32 cacheBlocks;		/*Num blocks in the cache */
-afs_int32 cacheFiles;	/*Optimal # of files in workstation cache */
+afs_int32 cacheFiles;		/*Optimal # of files in workstation cache */
+afs_int32 rwpct = 0;
+afs_int32 ropct = 0;
 afs_int32 cacheStatEntries;	/*Number of stat cache entries */
 char cacheBaseDir[1024];	/*Where the workstation AFS cache lives */
 char confDir[1024];		/*Where the workstation AFS configuration lives */
@@ -312,14 +281,13 @@ static int enable_dynroot = 0;	/* enable dynroot support */
 static int enable_fakestat = 0;	/* enable fakestat support */
 static int enable_backuptree = 0;	/* enable backup tree support */
 static int enable_nomount = 0;	/* do not mount */
+static int enable_splitcache = 0;
 #ifdef notdef
 static int inodes = 60;		/* VERY conservative, but has to be */
 #endif
-int afsd_dynamic_vcaches = 0;	/* Enable dynamic-vcache support */
 int afsd_verbose = 0;		/*Are we being chatty? */
 int afsd_debug = 0;		/*Are we printing debugging info? */
 int afsd_CloseSynch = 0;	/*Are closes synchronous or not? */
-int rxmaxmtu = 0;       /* Are we forcing a limit on the mtu? */
 
 #ifdef AFS_SGI62_ENV
 #define AFSD_INO_T ino64_t
@@ -337,10 +305,8 @@ int *dir_for_V = NULL;		/* Array: dir of each cache file.
 				 * -2: file exists in top-level
 				 * >=0: file exists in Dxxx
 				 */
-#ifndef AFS_CACHE_VNODE_PATH
 AFSD_INO_T *inode_for_V;	/* Array of inodes for desired
 				 * cache files */
-#endif
 int missing_DCacheFile = 1;	/*Is the DCACHEFILE missing? */
 int missing_VolInfoFile = 1;	/*Is the VOLINFOFILE missing? */
 int missing_CellInfoFile = 1;	/*Is the CELLINFOFILE missing? */
@@ -349,162 +315,6 @@ struct afs_cacheParams cparams;	/* params passed to cache manager */
 
 static int HandleMTab();
 
-#ifdef AFS_DARWIN_ENV
-static void
-afsd_sleep_callback(void * refCon, io_service_t service, 
-		    natural_t messageType, void * messageArgument )
-{
-    afs_int32 code;
-    
-    switch (messageType) {
-    case kIOMessageCanSystemSleep:
-	/* Idle sleep is about to kick in; can 
-	   prevent sleep by calling IOCancelPowerChange, otherwise 
-	   if we don't ack in 30s the system sleeps anyway */
-	
-	/* allow it */
-	IOAllowPowerChange(root_port, (long)messageArgument);
-	break;
-	
-    case kIOMessageSystemWillSleep:
-	/* The system WILL go to sleep. Ack or suffer delay */
-	
-	IOAllowPowerChange(root_port, (long)messageArgument);
-	break;
-	
-    case kIOMessageSystemWillRestart:
-	/* The system WILL restart. Ack or suffer delay */
-	
-	IOAllowPowerChange(root_port, (long)messageArgument);
-	break;
-	
-    case kIOMessageSystemWillPowerOn:
-    case kIOMessageSystemHasPoweredOn:
-	/* coming back from sleep */
-	
-	IOAllowPowerChange(root_port, (long)messageArgument);
-	break;
-	
-    default:
-	IOAllowPowerChange(root_port, (long)messageArgument);
-	break;
-    }
-}
-
-static void
-afsd_update_addresses(CFRunLoopTimerRef timer, void *info)
-{
-    /* parse multihomed address files */
-    afs_int32 addrbuf[MAXIPADDRS], maskbuf[MAXIPADDRS],
-	mtubuf[MAXIPADDRS];
-    char reason[1024];
-    afs_int32 code;
-
-    code =
-	parseNetFiles(addrbuf, maskbuf, mtubuf, MAXIPADDRS, reason,
-		      AFSDIR_CLIENT_NETINFO_FILEPATH,
-		      AFSDIR_CLIENT_NETRESTRICT_FILEPATH);
-
-    if (code > 0) {
-	/* Note we're refreshing */
-	code = code | 0x40000000;
-	call_syscall(AFSOP_ADVISEADDR, code, addrbuf, maskbuf, mtubuf);
-    } else
-	printf("ADVISEADDR: Error in specifying interface addresses:%s\n",
-	       reason);
-}
-
-/* This function is called when the system's ip addresses may have changed. */
-static void
-afsd_ipaddr_callback (SCDynamicStoreRef store, CFArrayRef changed_keys, void *info)
-{
-      CFRunLoopTimerRef timer;
-
-      timer = CFRunLoopTimerCreate (NULL, CFAbsoluteTimeGetCurrent () + 1.0,
-				    0.0, 0, 0, afsd_update_addresses, NULL);
-      CFRunLoopAddTimer (CFRunLoopGetCurrent (), timer,
-			 kCFRunLoopDefaultMode);
-      CFRelease (timer);
-}
-
-static void 
-afsd_event_cleanup(int signo) {
-    DNSSDState *query = &dnsstate;
-
-    CFRunLoopRemoveSource(CFRunLoopGetCurrent(), source, kCFRunLoopDefaultMode);
-    CFRelease (source);
-    IODeregisterForSystemPower(&iterator);
-    IOServiceClose(root_port);
-    IONotificationPortDestroy(notify);
-
-    exit(0);
-}
-
-/* Adapted from "Living in a Dynamic TCP/IP Environment" technote. */
-static Boolean
-afsd_install_events(void)
-{
-    SCDynamicStoreContext ctx = {0};
-    SCDynamicStoreRef store;
-
-    root_port = IORegisterForSystemPower(0,&notify,afsd_sleep_callback,&iterator);
-    
-    if (root_port) {
-	CFRunLoopAddSource(CFRunLoopGetCurrent(),
-			   IONotificationPortGetRunLoopSource(notify),
-			   kCFRunLoopDefaultMode);
-    }
-    
-    
-    store = SCDynamicStoreCreate (NULL,
-				  CFSTR ("AddIPAddressListChangeCallbackSCF"),
-				  afsd_ipaddr_callback, &ctx);
-    
-    if (store) {
-	const void *keys[1];
-	
-	/* Request IPV4 address change notification */
-	keys[0] = (SCDynamicStoreKeyCreateNetworkServiceEntity
-		   (NULL, kSCDynamicStoreDomainState,
-		    kSCCompAnyRegex, kSCEntNetIPv4));
-	
-#if 0
-	/* This should tell us when the hostname(s) change. do we care? */
-	keys[N] = SCDynamicStoreKeyCreateHostNames (NULL);
-#endif
-	
-	if (keys[0] != NULL) {
-	    CFArrayRef pattern_array;
-	    
-	    pattern_array = CFArrayCreate (NULL, keys, 1,
-					   &kCFTypeArrayCallBacks);
-	    
-	    if (pattern_array != NULL)
-	    {
-		SCDynamicStoreSetNotificationKeys (store, NULL, pattern_array);
-		source = SCDynamicStoreCreateRunLoopSource (NULL, store, 0);
-		
-		CFRelease (pattern_array);
-	    }
-	    
-	    if (keys[0] != NULL)
-		CFRelease (keys[0]);
-	}
-	
-	CFRelease (store); 
-    }
-    
-    if (source != NULL) {
-	CFRunLoopAddSource (CFRunLoopGetCurrent(),
-			    source, kCFRunLoopDefaultMode);
-    }
-    
-    signal(SIGTERM, afsd_event_cleanup);
-
-    CFRunLoopRun();
-}
-#endif
-
 /* ParseArgs is now obsolete, being handled by cmd */
 
 /*------------------------------------------------------------------------------
@@ -530,7 +340,8 @@ afsd_install_events(void)
   *	Sets globals.
   *---------------------------------------------------------------------------*/
 
-int ParseCacheInfoFile(void)
+int
+ParseCacheInfoFile(void)
 {
     static char rn[] = "ParseCacheInfoFile";	/*This routine's name */
     FILE *cachefd;		/*Descriptor for cache info file */
@@ -593,7 +404,7 @@ int ParseCacheInfoFile(void)
 	     tmd, tbd, tCacheBlocks);
     }
     if (!(cacheFlags & AFSCALL_INIT_MEMCACHE)) {
-	return(PartSizeOverflow(tbd, cacheBlocks));
+	return (PartSizeOverflow(tbd, cacheBlocks));
     }
 
     return (0);
@@ -609,10 +420,10 @@ int ParseCacheInfoFile(void)
  *	0 if everything went well,
  *	1 otherwise.
  */
-int PartSizeOverflow(char *path, int cs)
+int
+PartSizeOverflow(char *path, int cs)
 {
-  int bsize = -1;
-  afs_int64 totalblks, mint;
+    int bsize = -1, totalblks, mint;
 #if AFS_HAVE_STATVFS
     struct statvfs statbuf;
 
@@ -626,10 +437,10 @@ int PartSizeOverflow(char *path, int cs)
     totalblks = statbuf.f_blocks;
     bsize = statbuf.f_frsize;
 #if AFS_AIX51_ENV
-    if(strcmp(statbuf.f_basetype, "jfs")) {
-        fprintf(stderr, "Cache filesystem '%s' must be jfs (now %s)\n",
-                path, statbuf.f_basetype);
-        return 1;
+    if (strcmp(statbuf.f_basetype, "jfs")) {
+	fprintf(stderr, "Cache filesystem '%s' must be jfs (now %s)\n",
+		path, statbuf.f_basetype);
+	return 1;
     }
 #endif /* AFS_AIX51_ENV */
 
@@ -647,7 +458,7 @@ int PartSizeOverflow(char *path, int cs)
     bsize = statbuf.f_bsize;
 #endif
     if (bsize == -1)
-	return 0;		/* success */
+	return 0;		/* sucess */
 
     /* now free and totalblks are in fragment units, but we want them in 1K units */
     if (bsize >= 1024) {
@@ -661,7 +472,7 @@ int PartSizeOverflow(char *path, int cs)
 	printf
 	    ("Cache size (%d) must be less than 95%% of partition size (which is %d). Lower cache size\n",
 	     cs, mint);
-        return 1;
+	return 1;
     }
 
     return 0;
@@ -990,8 +801,6 @@ doSweepAFSCache(vFilesFound, directory, dirNum, maxDir)
 #ifdef AFS_SGI62_ENV
 	    printf("\tinode=%lld, reclen=%d, name='%s'\n", currp->d_ino,
 		   currp->d_reclen, currp->d_name);
-#elif defined(AFS_DFBSD_ENV)
-	    printf("\tinode=%d, name='%s'\n", currp->d_ino, currp->d_name);
 #else
 	    printf("\tinode=%d, reclen=%d, name='%s'\n", currp->d_ino,
 		   currp->d_reclen, currp->d_name);
@@ -1013,9 +822,7 @@ doSweepAFSCache(vFilesFound, directory, dirNum, maxDir)
 	     * file's inode, directory, and bump the number of files found
 	     * total and in this directory.
 	     */
-#ifndef AFS_CACHE_VNODE_PATH
 	    inode_for_V[vFileNum] = currp->d_ino;
-#endif
 	    dir_for_V[vFileNum] = dirNum;	/* remember this directory */
 
 	    if (!maxDir) {
@@ -1152,9 +959,7 @@ doSweepAFSCache(vFilesFound, directory, dirNum, maxDir)
 			   vFileNum);
 		else {
 		    struct stat statb;
-#ifndef AFS_CACHE_VNODE_PATH
 		    assert(inode_for_V[vFileNum] == (AFSD_INO_T) 0);
-#endif
 		    sprintf(vFilePtr, "D%d/V%d", thisDir, vFileNum);
 		    if (afsd_verbose)
 			printf("%s: Creating '%s'\n", rn, fullpn_VFile);
@@ -1165,9 +970,7 @@ doSweepAFSCache(vFilesFound, directory, dirNum, maxDir)
 		    if (CreateCacheFile(fullpn_VFile, &statb))
 			printf("%s: Can't create '%s'\n", rn, fullpn_VFile);
 		    else {
-#ifndef AFS_CACHE_VNODE_PATH
 			inode_for_V[vFileNum] = statb.st_ino;
-#endif
 			dir_for_V[vFileNum] = thisDir;
 			cache_dir_list[thisDir]++;
 			(*vFilesFound)++;
@@ -1269,12 +1072,8 @@ CheckCacheBaseDir(char *dir)
 	}
 	if (statfsbuf.f_type == 0x52654973) {	/* REISERFS_SUPER_MAGIC */
 	    return "cannot use reiserfs as cache partition";
-	} else if  (statfsbuf.f_type == 0x58465342) { /* XFS_SUPER_MAGIC */
+	} else if (statfsbuf.f_type == 0x58465342) {	/* XFS_SUPER_MAGIC */
 	    return "cannot use xfs as cache partition";
-	} else if (statfsbuf.f_type == 0x01021994) {    /* TMPFS_SUPER_MAGIC */
-            return "cannot use tmpfs as cache partition";
-        } else if (statfsbuf.f_type != 0xEF53) {
-            return "must use ext2 or ext3 for cache partition";
 	}
     }
 #endif
@@ -1395,8 +1194,8 @@ SweepAFSCache(vFilesFound)
     return doSweepAFSCache(vFilesFound, cacheBaseDir, -2, maxDir);
 }
 
-static int
-ConfigCell(struct afsconf_cell *aci, void *arock, struct afsconf_dir *adir)
+static
+ConfigCell(struct afsconf_cell *aci, char *arock, struct afsconf_dir *adir)
 {
     int isHomeCell;
     int i, code;
@@ -1428,7 +1227,7 @@ ConfigCell(struct afsconf_cell *aci, void *arock, struct afsconf_dir *adir)
 
 static
 ConfigCellAlias(struct afsconf_cellalias *aca,
-		void *arock, struct afsconf_dir *adir)
+		char *arock, struct afsconf_dir *adir)
 {
     /* push the alias into the kernel */
     call_syscall(AFSOP_ADDCELLALIAS, aca->aliasName, aca->realName);
@@ -1449,16 +1248,6 @@ AfsdbLookupHandler()
     kernelMsg[1] = 0;
     acellName[0] = '\0';
 
-#ifdef AFS_DARWIN_ENV
-    /* Fork the event handler also. */
-    code = fork();
-    if (code == 0) {
-	afsd_install_events();
-	exit(1);
-    } else if (code != -1) {
-	event_pid = code;
-    }
-#endif
     while (1) {
 	/* On some platforms you only get 4 args to an AFS call */
 	int sizeArg = ((sizeof acellName) << 16) | (sizeof kernelMsg);
@@ -1488,9 +1277,7 @@ AfsdbLookupHandler()
 	    acellName[sizeof(acellName) - 1] = '\0';
 	}
     }
-#ifdef AFS_DARWIN_ENV
-    kill(event_pid, SIGTERM);
-#endif
+
     exit(1);
 }
 #endif
@@ -1521,7 +1308,7 @@ AfsdbLookupHandler()
 #endif
 #endif
 
-mainproc(struct cmd_syndesc *as, void *arock)
+mainproc(struct cmd_syndesc *as, char *arock)
 {
     static char rn[] = "afsd";	/*Name of this routine */
     afs_int32 code;		/*Result of fork() */
@@ -1611,7 +1398,8 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	/* -chunksize */
 	chunkSize = atoi(as->parms[12].items->data);
 	if (chunkSize < 0 || chunkSize > 30) {
-	    printf("afsd:invalid chunk size (not in range 0-30), using default\n");
+	    printf
+		("afsd:invalid chunk size (not in range 0-30), using default\n");
 	    chunkSize = 0;
 	}
     }
@@ -1728,47 +1516,37 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	enable_rxbind = 1;
     }
     if (as->parms[32].items) {
-       /* -settime */
-       cacheSetTime = TRUE;
+	/* -settime */
+	cacheSetTime = TRUE;
     }
 
     /* set rx_extraPackets */
     if (as->parms[33].items) {
 	/* -rxpck */
 	int rxpck = atoi(as->parms[33].items->data);
-	printf("afsd: set rxpck = %d\n",rxpck);
+	printf("afsd: set rxpck = %d\n", rxpck);
 	code = call_syscall(AFSOP_SET_RXPCK, rxpck);
 	if (code) {
-	printf("afsd: failed to set rxpck\n");
-	exit(1);
+	    printf("afsd: failed to set rxpck\n");
+	    exit(1);
 	}
     }
-    
     if (as->parms[34].items) {
-#ifdef AFS_MAXVCOUNT_ENV
-       /* -disable-dynamic-vcaches */
-       afsd_dynamic_vcaches = FALSE;
-#else
-       printf("afsd: Error toggling flag, dynamically allocated vcaches not supported on your platform\n");
-       exit(1);
-#endif
+	char *c;
+	if (!as->parms[34].items->data ||
+	    ((c = strchr(as->parms[34].items->data, '/')) == NULL))
+	    printf
+		("ignoring splitcache (specify as RW/RO percentages: 60/40)\n");
+	else {
+	    ropct = atoi((char *)c + 1);
+	    *c = '\0';
+	    rwpct = atoi((char *)as->parms[30].items->data);
+	    if ((rwpct != 0) && (ropct != 0) && (ropct + rwpct == 100)) {
+		/* -splitcache */
+		enable_splitcache = 1;
+	    }
+	}
     }
-#ifdef AFS_MAXVCOUNT_ENV
-    else {
-       /* -dynamic-vcaches */
-       afsd_dynamic_vcaches = TRUE;
-    }
-
-    if (afsd_verbose)
-    printf("afsd: %s dynamically allocated vcaches\n", ( afsd_dynamic_vcaches ? "enabling" : "disabling" ));
-#endif
-
-    /* set -rxmaxmtu */
-    if (as->parms[35].items) {
-        /* -rxmaxmtu */
-        rxmaxmtu = atoi(as->parms[35].items->data);
-    }
-
     /*
      * Pull out all the configuration info for the workstation's AFS cache and
      * the cellular community we're willing to let our users see.
@@ -1905,19 +1683,12 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	    if (afsd_verbose)
 		printf("%s: cacheFiles autotuned to %d\n", rn, cacheFiles);
 	}
-#if 0
-       /* This actually needs to
-          1) use powers of 2
-          2) not second-guess when a chunksize comes from the command line
-          3) be less, um, small. 2^2?? 
-       */
 	/* Sanity check chunkSize */
 	i = max(cacheBlocks / 1000, cacheBlocks / cacheFiles);
 	chunkSize = min(chunkSize, i);
 	chunkSize = max(chunkSize, 2);
 	if (afsd_verbose)
 	    printf("%s: chunkSize autotuned to %d\n", rn, chunkSize);
-#endif
 
 	if (!sawDCacheSize) {
 	    dCacheSize = cacheFiles / 2;
@@ -1944,7 +1715,6 @@ mainproc(struct cmd_syndesc *as, void *arock)
 		   cacheStatEntries);
     }
 
-#ifndef AFS_CACHE_VNODE_PATH
     /*
      * Create and zero the inode table for the desired cache files.
      */
@@ -1959,7 +1729,6 @@ mainproc(struct cmd_syndesc *as, void *arock)
     if (afsd_debug)
 	printf("%s: %d inode_for_V entries at 0x%x, %d bytes\n", rn,
 	       cacheFiles, inode_for_V, (cacheFiles * sizeof(AFSD_INO_T)));
-#endif
 
     /*
      * Set up all the pathnames we'll need for later.
@@ -1970,14 +1739,14 @@ mainproc(struct cmd_syndesc *as, void *arock)
     sprintf(fullpn_VFile, "%s/", cacheBaseDir);
     vFilePtr = fullpn_VFile + strlen(fullpn_VFile);
 
-    if  (!(cacheFlags & AFSCALL_INIT_MEMCACHE) && (fsTypeMsg = CheckCacheBaseDir(cacheBaseDir))) {
-#ifdef AFS_SUN5_ENV
-        printf("%s: WARNING: Cache dir check failed (%s)\n", rn, fsTypeMsg);
-#else
-	printf("%s: ERROR: Cache dir check failed (%s)\n", rn, fsTypeMsg);
-	exit(1);
-#endif
+    if (!(cacheFlags & AFSCALL_INIT_MEMCACHE)
+	&& (fsTypeMsg = CheckCacheBaseDir(cacheBaseDir))) {
+	printf("%s: WARNING: Cache dir check failed (%s)\n", rn, fsTypeMsg);
     }
+#if 0
+    fputs(AFS_GOVERNMENT_MESSAGE, stdout);
+    fflush(stdout);
+#endif
 
     /*
      * Set up all the kernel processes needed for AFS.
@@ -2074,7 +1843,7 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	     */
 	    if (daemon(0, 0) == -1) {
 		printf("Error starting AFSDB lookup handler: %s\n",
-			strerror(errno));
+		       strerror(errno));
 		exit(1);
 	    }
 	    AfsdbLookupHandler();
@@ -2106,11 +1875,17 @@ mainproc(struct cmd_syndesc *as, void *arock)
     cparams.chunkSize = chunkSize;
     cparams.setTimeFlag = cacheSetTime;
     cparams.memCacheFlag = cacheFlags;
-    cparams.dynamic_vcaches = afsd_dynamic_vcaches;
 #ifdef notdef
     cparams.inodes = inodes;
 #endif
     call_syscall(AFSOP_CACHEINIT, &cparams);
+
+    /* do it before we init the cache inodes */
+    if (enable_splitcache) {
+	call_syscall(AFSOP_BUCKETPCT, 1, rwpct);
+	call_syscall(AFSOP_BUCKETPCT, 2, ropct);
+    }
+
     if (afsd_CloseSynch)
 	call_syscall(AFSOP_CLOSEWAIT);
 
@@ -2141,23 +1916,6 @@ mainproc(struct cmd_syndesc *as, void *arock)
 		     rn, vFilesFound, cacheFiles, cacheIteration);
 	} while ((vFilesFound < cacheFiles)
 		 && (cacheIteration < MAX_CACHE_LOOPS));
-#ifdef AFS_CACHE_VNODE_PATH
-	if (afsd_debug)
-	    printf
-		("%s: Calling AFSOP_CACHEBASEDIR with '%s'\n",
-		 rn, cacheBaseDir);
-	call_syscall(AFSOP_CACHEBASEDIR, cacheBaseDir);
-	if (afsd_debug)
-	    printf
-		("%s: Calling AFSOP_CACHEDIRS with %d dirs\n",
-		 rn, nFilesPerDir);
-	call_syscall(AFSOP_CACHEDIRS, nFilesPerDir);
-	if (afsd_debug)
-	    printf
-		("%s: Calling AFSOP_CACHEFILES with %d files\n",
-		 rn, cacheFiles);
-	call_syscall(AFSOP_CACHEFILES, cacheFiles);
-#endif
     } else if (afsd_verbose)
 	printf("%s: Using memory cache, not swept\n", rn);
 
@@ -2181,14 +1939,6 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	       fullpn_CellInfoFile);
     call_syscall(AFSOP_CELLINFO, fullpn_CellInfoFile);
 
-    if (rxmaxmtu) {
-    if (afsd_verbose)
-        printf("%s: Setting rxmaxmtu in kernel = %d\n", rn, rxmaxmtu);
-    code = call_syscall(AFSOP_SET_RXMAXMTU, rxmaxmtu);
-    if (code)
-        printf("%s: Error seting rxmaxmtu\n", rn);
-    }
-
     if (enable_dynroot) {
 	if (afsd_verbose)
 	    printf("%s: Enabling dynroot support in kernel.\n", rn);
@@ -2297,10 +2047,9 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	printf("%s: Calling AFSOP_VOLUMEINFO: volume info file is '%s'\n", rn,
 	       fullpn_VolInfoFile);
     /* once again, meaningless for a memory-based cache. */
-    if (!(cacheFlags & AFSCALL_INIT_MEMCACHE)) 
+    if (!(cacheFlags & AFSCALL_INIT_MEMCACHE))
 	call_syscall(AFSOP_VOLUMEINFO, fullpn_VolInfoFile);
 
-#ifndef AFS_CACHE_VNODE_PATH
     /*
      * Give the kernel the names of the AFS files cached on the workstation's
      * disk.
@@ -2319,7 +2068,7 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	    call_syscall(AFSOP_CACHEINODE, inode_for_V[currVFile]);
 #endif
 	}
-#endif
+
 
     /*end for */
     /*
@@ -2377,7 +2126,8 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	if ((mount("AFS", cacheMountDir, mountFlags, "afs", NULL, 0)) < 0) {
 #elif defined(AFS_SGI_ENV)
 	mountFlags = MS_FSS;
-	if ((mount(MOUNT_AFS, cacheMountDir, mountFlags, (caddr_t) MOUNT_AFS)) < 0) {
+	if ((mount(MOUNT_AFS, cacheMountDir, mountFlags, (caddr_t) MOUNT_AFS))
+	    < 0) {
 #elif defined(AFS_LINUX20_ENV)
 	if ((mount("AFS", cacheMountDir, MOUNT_AFS, 0, NULL)) < 0) {
 #else
@@ -2483,12 +2233,14 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-nomount", CMD_FLAG, CMD_OPTIONAL, "Do not mount AFS");
     cmd_AddParm(ts, "-backuptree", CMD_FLAG, CMD_OPTIONAL,
 		"Prefer backup volumes for mointpoints in backup volumes");
-    cmd_AddParm(ts, "-rxbind", CMD_FLAG, CMD_OPTIONAL, "Bind the Rx socket (one interface only)");
-    cmd_AddParm(ts, "-settime", CMD_FLAG, CMD_OPTIONAL,
-               "set the time");
-    cmd_AddParm(ts, "-rxpck", CMD_SINGLE, CMD_OPTIONAL, "set rx_extraPackets to this value");
-    cmd_AddParm(ts, "-disable-dynamic-vcaches", CMD_FLAG, CMD_OPTIONAL, "disable stat/vcache cache growing as needed");
-    cmd_AddParm(ts, "-rxmaxmtu", CMD_SINGLE, CMD_OPTIONAL, "set rx max MTU to use");
+    cmd_AddParm(ts, "-rxbind", CMD_FLAG, CMD_OPTIONAL,
+		"Bind the Rx socket (one interface only)");
+    cmd_AddParm(ts, "-settime", CMD_FLAG, CMD_OPTIONAL, "set the time");
+    cmd_AddParm(ts, "-rxpck", CMD_SINGLE, CMD_OPTIONAL,
+		"set rx_extraPackets to this value");
+    cmd_AddParm(ts, "-splitcache", CMD_SINGLE, CMD_OPTIONAL,
+		"Percentage RW versus RO in cache (specify as 60/40)");
+
     return (cmd_Dispatch(argc, argv));
 }
 
@@ -2566,7 +2318,6 @@ HandleMTab()
 #endif /* AFS_SUN5_ENV */
 #endif /* unreasonable systems */
 #ifdef AFS_DARWIN_ENV
-#ifndef AFS_DARWIN100_ENV
     mach_port_t diskarb_port;
     kern_return_t status;
 
@@ -2579,23 +2330,22 @@ HandleMTab()
     }
 
     return status;
-#endif
 #endif /* AFS_DARWIN_ENV */
     return 0;
 }
 
 #if !defined(AFS_SGI_ENV) && !defined(AFS_AIX32_ENV)
-int
-call_syscall(long param1, long param2, long param3, long param4, long param5, 
-	     long param6, long param7)
+
+call_syscall(param1, param2, param3, param4, param5, param6, param7)
+     long param1, param2, param3, param4, param5, param6, param7;
 {
     int error;
 #ifdef AFS_LINUX20_ENV
     long eparm[4];
     struct afsprocdata syscall_data;
-    int fd = open(PROC_SYSCALL_FNAME,O_RDWR);
+    int fd = open(PROC_SYSCALL_FNAME, O_RDWR);
     if (fd < 0)
-	fd = open(PROC_SYSCALL_ARLA_FNAME,O_RDWR);
+	fd = open(PROC_SYSCALL_ARLA_FNAME, O_RDWR);
     eparm[0] = param4;
     eparm[1] = param5;
     eparm[2] = param6;
@@ -2608,58 +2358,29 @@ call_syscall(long param1, long param2, long param3, long param4, long param5,
     syscall_data.param2 = param2;
     syscall_data.param3 = param3;
     syscall_data.param4 = param4;
-    if(fd > 0) {
-       error = ioctl(fd, VIOC_SYSCALL, &syscall_data);
-       close(fd);
-    }
-    else
+    if (fd > 0) {
+	error = ioctl(fd, VIOC_SYSCALL, &syscall_data);
+	close(fd);
+    } else
 #endif
 #ifdef AFS_DARWIN80_ENV
     struct afssysargs syscall_data;
-    void *ioctldata;
     int fd = open(SYSCALL_DEV_FNAME,O_RDWR);
-    int syscallnum, is64 = 0;
-#ifdef AFS_DARWIN100_ENV
-    struct afssysargs64 syscall64_data;
-    if (sizeof(param1) == 8) {
-	syscallnum = VIOC_SYSCALL64;
-	is64 = 1;
-	ioctldata = &syscall64_data;
-	syscall64_data.syscall = (int)AFSCALL_CALL;
-	syscall64_data.param1 = param1;
-	syscall64_data.param2 = param2;
-	syscall64_data.param3 = param3;
-	syscall64_data.param4 = param4;
-	syscall64_data.param5 = param5;
-	syscall64_data.param6 = param6;
-    } else {
-#endif
-	syscallnum = VIOC_SYSCALL;
-	ioctldata = &syscall_data;
-	syscall_data.syscall = AFSCALL_CALL;
-	syscall_data.param1 = param1;
-	syscall_data.param2 = param2;
-	syscall_data.param3 = param3;
-	syscall_data.param4 = param4;
-	syscall_data.param5 = param5;
-	syscall_data.param6 = param6;
-#ifdef AFS_DARWIN100_ENV
-    }
-#endif
+    syscall_data.syscall = AFSCALL_CALL;
+    syscall_data.param1 = param1;
+    syscall_data.param2 = param2;
+    syscall_data.param3 = param3;
+    syscall_data.param4 = param4;
+    syscall_data.param5 = param5;
+    syscall_data.param6 = param6;
     if(fd >= 0) {
-	error = ioctl(fd, syscallnum, ioctldata);
-	close(fd);
+       error = ioctl(fd, VIOC_SYSCALL, &syscall_data);
+       close(fd);
     } else {
-	error = -1;
-    }
-    if (!error) {
-#ifdef AFS_DARWIN100_ENV
-	if (is64)
-	    error=syscall64_data.retval;
-	else
-#endif
-	    error=syscall_data.retval;
+       error = -1;
     }
+    if (!error)
+      error=syscall_data.retval;
 #else
     error =
 	syscall(AFS_SYSCALL, AFSCALL_CALL, param1, param2, param3, param4,
