@@ -1,4 +1,3 @@
-
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
@@ -112,6 +111,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/ptserver/ptserver.c,v 1.24 2006/02/01 16:09:19 shadow Exp $");
 
 #include <afs/stds.h>
 #ifdef	AFS_AIX32_ENV
@@ -128,7 +129,13 @@
 #include <netdb.h>
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
@@ -141,7 +148,6 @@
 #include "error_macros.h"
 #include "afs/audit.h"
 #include <afs/afsutil.h>
-#include <afs/com_err.h>
 
 
 /* make	all of these into a structure if you want */
@@ -159,14 +165,7 @@ extern int afsconf_CheckAuth();
 int pr_realmNameLen;
 char *pr_realmName;
 
-int debuglevel = 0;
 int restricted = 0;
-int rxMaxMTU = -1;
-int rxBind = 0;
-int rxkadDisableDotCheck = 0;
-
-#define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
-afs_uint32 SHostAddrs[ADDRSPERSITE];
 
 static struct afsconf_cell info;
 
@@ -222,7 +221,6 @@ main(int argc, char **argv)
     int kerberosKeys;		/* set if found some keys */
     int lwps = 3;
     char clones[MAXHOSTSPERCELL];
-    afs_uint32 host = htonl(INADDR_ANY);
 
     const char *pr_dbaseName;
     char *whoami = "ptserver";
@@ -277,14 +275,7 @@ main(int argc, char **argv)
 	int alen;
 	lcstring(arg, argv[a], sizeof(arg));
 	alen = strlen(arg);
-	if (strcmp(argv[a], "-d") == 0) {
-	    if ((a + 1) >= argc) {
-		fprintf(stderr, "missing argument for -d\n"); 
-		return -1; 
-	    }
-	    debuglevel = atoi(argv[++a]);
-	    LogLevel = debuglevel;
-	} else if ((strncmp(arg, "-database", alen) == 0)
+	if ((strncmp(arg, "-database", alen) == 0)
 	    || (strncmp(arg, "-db", alen) == 0)) {
 	    pr_dbaseName = argv[++a];	/* specify a database */
 	} else if (strncmp(arg, "-p", alen) == 0) {
@@ -313,12 +304,6 @@ main(int argc, char **argv)
 	else if (strncmp(arg, "-restricted", alen) == 0) {
 	    restricted = 1;
 	}
-	else if (strncmp(arg, "-rxbind", alen) == 0) {
-	    rxBind = 1;
-	}
-	else if (strncmp(arg, "-allow-dotted-principals", alen) == 0) {
-	    rxkadDisableDotCheck = 1;
-	}
 	else if (strncmp(arg, "-enable_peer_stats", alen) == 0) {
 	    rx_enablePeerRPCStats();
 	} else if (strncmp(arg, "-enable_process_stats", alen) == 0) {
@@ -334,25 +319,36 @@ main(int argc, char **argv)
 	}
 #endif
 	else if (strncmp(arg, "-auditlog", alen) == 0) {
+	    int tempfd, flags;
+	    FILE *auditout;
+	    char oldName[MAXPATHLEN];
 	    char *fileName = argv[++a];
 
-            osi_audit_file(fileName);
-            osi_audit(PTS_StartEvent, 0, AUD_END);
+#ifndef AFS_NT40_ENV
+	    struct stat statbuf;
+
+	    if ((lstat(fileName, &statbuf) == 0) 
+		&& (S_ISFIFO(statbuf.st_mode))) {
+		flags = O_WRONLY | O_NONBLOCK;
+	    } else 
+#endif
+	    {
+		strcpy(oldName, fileName);
+		strcat(oldName, ".old");
+		renamefile(fileName, oldName);
+		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	    }
+	    tempfd = open(fileName, flags, 0666);
+	    if (tempfd > -1) {
+		auditout = fdopen(tempfd, "a");
+		if (auditout) {
+		    osi_audit_file(auditout);
+		    osi_audit(PTS_StartEvent, 0, AUD_END);
+		} else
+		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	}
-	else if (!strncmp(arg, "-rxmaxmtu", alen)) {
-	    if ((a + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n");
-		PT_EXIT(1);
-	    }
-	    rxMaxMTU = atoi(argv[++a]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
-		 (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%d\n",
-			rxMaxMTU, RX_MIN_PACKET_SIZE,
-			RX_MAX_PACKET_DATA_SIZE);
-		PT_EXIT(1);
-	    }
-	} 
 	else if (*arg == '-') {
 	    /* hack in help flag support */
 
@@ -360,40 +356,36 @@ main(int argc, char **argv)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: ptserver [-database <db path>] "
 		   "[-auditlog <log path>] "
-		   "[-syslog[=FACILITY]] [-d <debug level>] "
+		   "[-syslog[=FACILITY]] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-groupdepth <depth>] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-allow-dotted-principals] "
+		   "[-restricted] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-default_access default_user_access default_group_access] "
 		   "[-help]\n");
 #else /* AFS_NT40_ENV */
 	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] [-d <debug level>] "
-		   "[-p <number of processes>] [-rebuild] [-rxbind] "
-		   "[-allow-dotted-principals] "
+		   "[-auditlog <log path>] "
+		   "[-p <number of processes>] [-rebuild] "
 		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-restricted] "
 		   "[-groupdepth <depth>] " "[-help]\n");
 #endif
 #else
 #ifndef AFS_NT40_ENV
 	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] [-d <debug level>] "
+		   "[-auditlog <log path>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-allow-dotted-principals] "
+		   "[-restricted] "
 		   "[-help]\n");
 #else /* AFS_NT40_ENV */
 	    printf("Usage: ptserver [-database <db path>] "
-		   "[-auditlog <log path>] [-d <debug level>] "
+		   "[-auditlog <log path>] "
 		   "[-default_access default_user_access default_group_access] "
-		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
-		   "[-allow-dotted-principals] "
+		   "[-restricted] "
 		   "[-p <number of processes>] [-rebuild] " "[-help]\n");
 #endif
 #endif
@@ -444,7 +436,7 @@ main(int argc, char **argv)
     code =
 	afsconf_GetExtendedCellInfo(prdir, NULL, "afsprot", &info, &clones);
     if (code) {
-	afs_com_err(whoami, code, "Couldn't get server list");
+	com_err(whoami, code, "Couldn't get server list");
 	PT_EXIT(2);
     }
     pr_realmName = info.name;
@@ -454,7 +446,7 @@ main(int argc, char **argv)
     /* get keys */
     code = afsconf_GetKey(prdir, 999, &tkey);
     if (code) {
-	afs_com_err(whoami, code,
+	com_err(whoami, code,
 		"couldn't get bcrypt keys from key file, ignoring.");
     }
 #endif
@@ -486,33 +478,11 @@ main(int argc, char **argv)
      * and the header are in separate Ubik buffers then 120 buffers may be
      * required. */
     ubik_nBuffers = 120 + /*fudge */ 40;
-
-    if (rxBind) {
-	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
-	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
-	    AFSDIR_SERVER_NETINFO_FILEPATH) {
-	    char reason[1024];
-	    ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-					   ADDRSPERSITE, reason,
-					   AFSDIR_SERVER_NETINFO_FILEPATH,
-					   AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-	} else 
-#endif
-	{
-	    ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
-	}
-	if (ccode == 1) {
-	    host = SHostAddrs[0];
-	    rx_InitHost(host, htons(AFSCONF_PROTPORT));
-	}
-    }
-
     code =
-	ubik_ServerInitByInfo(myHost, htons(AFSCONF_PROTPORT), &info, clones,
+	ubik_ServerInitByInfo(myHost, htons(AFSCONF_PROTPORT), &info, &clones,
 			      pr_dbaseName, &dbase);
     if (code) {
-	afs_com_err(whoami, code, "Ubik init failed");
+	com_err(whoami, code, "Ubik init failed");
 	PT_EXIT(2);
     }
 #if defined(SUPERGROUPS)
@@ -523,22 +493,14 @@ main(int argc, char **argv)
     sc[1] = 0;
     if (kerberosKeys) {
 	sc[2] = rxkad_NewServerSecurityObject(0, prdir, afsconf_GetKey, NULL);
-#ifdef USE_RXKAD_KEYTAB
-	if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
-	    rxkad_BindKeytabDecrypt(sc[2]);
-#endif
     } else
 	sc[2] = sc[0];
 
     /* Disable jumbograms */
     rx_SetNoJumbo();
 
-    if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
-    }
-
     tservice =
-	rx_NewServiceHost(host, 0, PRSRV, "Protection Server", sc, 3,
+	rx_NewService(0, PRSRV, "Protection Server", sc, 3,
 		      PR_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	fprintf(stderr, "ptserver: Could not create new rx service.\n");
@@ -546,14 +508,9 @@ main(int argc, char **argv)
     }
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, lwps);
-    if (rxkadDisableDotCheck) {
-        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-                                    NULL);
-    }
 
     tservice =
-	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", sc, 3,
+	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", sc, 3,
 		      RXSTATS_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	fprintf(stderr, "ptserver: Could not create new rx service.\n");
