@@ -14,16 +14,19 @@
  * Its value should be as large as the maximum file descriptor limit we
  * are likely to run into on any platform.  Right now, that is 65536
  * which is the default hard fd limit on Solaris 9 */
+#if !defined(_WIN32) && !defined(KERNEL)
 #define FD_SETSIZE 65536
- 
+#endif
+
 #include <afsconfig.h>
 #ifdef KERNEL
-#include "../afs/param.h"
+#include "afs/param.h"
 #else
 #include <afs/param.h>
 #endif
 
-RCSID("$Header: /cvs/openafs/src/rx/rx_globals.c,v 1.4.4.1 2004/10/18 05:53:13 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/rx/rx_globals.c,v 1.9 2004/12/01 23:36:49 shadow Exp $");
 
 /* Enable data initialization when the header file is included */
 #define INIT(stuff) = stuff
@@ -35,9 +38,9 @@ RCSID("$Header: /cvs/openafs/src/rx/rx_globals.c,v 1.4.4.1 2004/10/18 05:53:13 s
 
 #ifdef KERNEL
 #ifndef UKERNEL
-#include "../h/types.h"
+#include "h/types.h"
 #else /* !UKERNEL */
-#include	"../afs/sysincludes.h"
+#include	"afs/sysincludes.h"
 #endif /* UKERNEL */
 #endif /* KERNEL */
 
