@@ -14,6 +14,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.55 2006/01/18 16:03:28 rees Exp $");
 
 #include "rx/rx_kcommon.h"
 
@@ -122,7 +124,6 @@ rxi_GetUDPSocket(u_short port)
     return rxi_GetHostUDPSocket(htonl(INADDR_ANY), port);
 }
 
-#if !defined(AFS_LINUX26_ENV)
 void
 osi_Panic(msg, a1, a2, a3)
      char *msg;
@@ -130,8 +131,9 @@ osi_Panic(msg, a1, a2, a3)
     if (!msg)
 	msg = "Unknown AFS panic";
 
-    printf(msg, a1, a2, a3);
-#ifdef AFS_LINUX20_ENV
+    dpf((msg, a1, a2, a3));
+#ifdef AFS_LINUX24_ENV
+    printk("AFS BUG at %s\n", msg); 
     * ((char *) 0) = 0; 
 #else
     panic(msg);
@@ -260,19 +262,12 @@ osi_AssertFailK(const char *expr, const char *file, int line)
 
     osi_Panic(buf);
 }
-#else
-void
-osi_AssertFailK(const char *expr, const char *file, int line)
-{
-    printk(KERN_CRIT "assertion failed: %s, file: %s, line: %d\n", expr, file, line);
-}
-#endif
 
 #ifndef UKERNEL
 /* This is the server process request loop. Kernel server
  * processes never become listener threads */
-void *
-rx_ServerProc(void *unused)
+void
+rx_ServerProc(void)
 {
     int threadID;
 
@@ -289,8 +284,6 @@ rx_ServerProc(void *unused)
 #ifdef RX_ENABLE_LOCKS
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
-
-    return NULL;
 }
 #endif /* !UKERNEL */
 
@@ -387,7 +380,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     if (i == -1) {
 	pp->timeout.sec = 3;
 	/* pp->timeout.usec = 0; */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
+	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
     } else {
 	pp->timeout.sec = 2;
 	/* pp->timeout.usec = 0; */
@@ -403,14 +396,10 @@ rxi_InitPeerParams(register struct rx_peer *pp)
 		pp->ifMTU = rxmtu;
 	}
     } else {			/* couldn't find the interface, so assume the worst */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
+	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
     }
 #else /* AFS_USERSPACE_IP_ADDR */
-#ifdef AFS_DARWIN80_ENV
-    ifnet_t ifn;
-#else
-    struct ifnet *ifn;
-#endif
+    AFS_IFNET_T ifn;
 
 #if !defined(AFS_SGI62_ENV)
     if (numMyNetAddrs == 0)
@@ -439,7 +428,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     } else {			/* couldn't find the interface, so assume the worst */
 	pp->timeout.sec = 3;
 	/* pp->timeout.usec = 0; */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
+	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
     }
 #endif /* else AFS_USERSPACE_IP_ADDR */
 #else /* AFS_SUN5_ENV */
@@ -448,7 +437,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     if (mtu <= 0) {
 	pp->timeout.sec = 3;
 	/* pp->timeout.usec = 0; */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
+	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
     } else {
 	pp->timeout.sec = 2;
 	/* pp->timeout.usec = 0; */
@@ -464,7 +453,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
 		pp->ifMTU = rxmtu;
 	}
     } else {			/* couldn't find the interface, so assume the worst */
-	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE,rx_MyMaxSendSize);
+	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
     }
 #endif /* AFS_SUN5_ENV */
 #else /* ADAPT_MTU */
@@ -477,7 +466,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     pp->natMTU = MIN(pp->ifMTU, OLD_MAX_PACKET_SIZE);
     pp->ifDgramPackets =
 	MIN(rxi_nDgramPackets,
-	    rxi_AdjustDgramPackets(rxi_nSendFrags, pp->ifMTU));
+	    rxi_AdjustDgramPackets(RX_MAX_FRAGS, pp->ifMTU));
     pp->maxDgramPackets = 1;
 
     /* Initialize slow start parameters */
@@ -524,7 +513,7 @@ shutdown_rxkernel(void)
 int
 rxi_GetcbiInfo(void)
 {
-    int i, j, different = 0, num = ADDRSPERSITE;
+    int i, j, different = 0;
     int rxmtu, maxmtu;
     afs_uint32 ifinaddr;
     afs_uint32 addrs[ADDRSPERSITE];
@@ -533,9 +522,7 @@ rxi_GetcbiInfo(void)
     memset((void *)addrs, 0, sizeof(addrs));
     memset((void *)mtus, 0, sizeof(mtus));
 
-    if (afs_cb_interface.numberOfInterfaces < num)
-	num = afs_cb_interface.numberOfInterfaces;
-    for (i = 0; i < num; i++) {
+    for (i = 0; i < afs_cb_interface.numberOfInterfaces; i++) {
 	if (!afs_cb_interface.mtu[i])
 	    afs_cb_interface.mtu[i] = htonl(1500);
 	rxmtu = (ntohl(afs_cb_interface.mtu[i]) - RX_IPUDP_SIZE);
@@ -641,13 +628,11 @@ rxi_GetIFInfo(void)
     afs_uint32 ifinaddr;
 #if defined(AFS_DARWIN80_ENV)
     errno_t t;
-    unsigned int count;
-    int cnt=0, m, j;
-    ifaddr_t *ifads;
-    ifnet_t *ifn;
+    int cnt=0;
+    ifaddr_t *ifads, ifad;
+    register ifnet_t ifn;
     struct sockaddr sout;
     struct sockaddr_in *sin;
-    struct in_addr pin;
 #else
     struct ifaddr *ifad;	/* ifnet points to a if_addrlist of ifaddrs */
     register struct ifnet *ifn;
@@ -657,38 +642,31 @@ rxi_GetIFInfo(void)
     memset(mtus, 0, sizeof(mtus));
 
 #if defined(AFS_DARWIN80_ENV)
-    if (!ifnet_list_get(AF_INET, &ifn, &count)) {
-	for (m = 0; m < count; m++) {
-	    if (!ifnet_get_address_list(ifn[m], &ifads)) {
-		for (j = 0; ifads[j] != NULL && cnt < ADDRSPERSITE; j++) {
-		    if ((t = ifaddr_address(ifads[j], &sout, sizeof(struct sockaddr))) == 0) {
-			sin = (struct sockaddr_in *)&sout;
-			rxmtu = ifnet_mtu(ifaddr_ifnet(ifads[j])) - RX_IPUDP_SIZE;
-			ifinaddr = ntohl(sin->sin_addr.s_addr);
-			if (myNetAddrs[i] != ifinaddr) {
-			    different++;
-			}
-			mtus[i] = rxmtu;
-			rxmtu = rxi_AdjustIfMTU(rxmtu);
-			maxmtu =
-			    rxmtu * rxi_nRecvFrags +
-			    ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
-			maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
-			addrs[i++] = ifinaddr;
-			if ((ifinaddr != 0x7f000001) && 
-			    (maxmtu > rx_maxReceiveSize)) {
-			    rx_maxReceiveSize = 
-				MIN(RX_MAX_PACKET_SIZE, maxmtu);
-			    rx_maxReceiveSize =
-				MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
-			}
-			cnt++;
-		    }
-		}
-		ifnet_free_address_list(ifads);
+    t = ifnet_get_address_list_family(NULL, &ifads, AF_INET);
+    if (t == 0) {
+	rxmtu = ifnet_mtu(ifn) - RX_IPUDP_SIZE;
+	while((ifads[cnt] != NULL) && cnt < ADDRSPERSITE) {
+	    t = ifaddr_address(ifads[cnt], &sout, sizeof(sout));
+	    sin = (struct sockaddr_in *)&sout;
+	    ifinaddr = ntohl(sin->sin_addr.s_addr);
+	    if (myNetAddrs[i] != ifinaddr) {
+		different++;
 	    }
+	    mtus[i] = rxmtu;
+	    rxmtu = rxi_AdjustIfMTU(rxmtu);
+	    maxmtu =
+		rxmtu * rxi_nRecvFrags +
+		((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
+	    maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
+	    addrs[i++] = ifinaddr;
+	    if ((ifinaddr != 0x7f000001) && (maxmtu > rx_maxReceiveSize)) {
+		rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxmtu);
+		rx_maxReceiveSize =
+		    MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
+	    }
+	    cnt++;
 	}
-	ifnet_list_free(ifn);
+	ifnet_free_address_list(ifads);
     }
 #else
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
@@ -744,10 +722,10 @@ rxi_GetIFInfo(void)
     rx_maxJumboRecvSize = MAX(rx_maxJumboRecvSize, rx_maxReceiveSize);
 
     if (different) {
-	int l;
-	for (l = 0; l < i; l++) {
-	    myNetMTUs[l] = mtus[l];
-	    myNetAddrs[l] = addrs[l];
+	int j;
+	for (j = 0; j < i; j++) {
+	    myNetMTUs[j] = mtus[j];
+	    myNetAddrs[j] = addrs[j];
 	}
     }
     return different;
@@ -773,8 +751,6 @@ rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp)
     s.sin_addr.s_addr = addr;
 #ifdef AFS_DARWIN80_ENV
     ifad = ifaddr_withnet((struct sockaddr *)&s);
-#elif AFS_OBSD46_ENV
-    ifad = ifa_ifwithnet((struct sockaddr *)&s, 0);
 #else
     ifad = ifa_ifwithnet((struct sockaddr *)&s);
 #endif
@@ -996,12 +972,10 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     }
     nam->m_len = sizeof(myaddr);
     memcpy(mtod(nam, caddr_t), &myaddr, sizeof(myaddr));
-#if defined(AFS_SGI65_ENV)
+#ifdef AFS_SGI65_ENV
     BHV_PDATA(&bhv) = (void *)newSocket;
     code = sobind(&bhv, nam);
     m_freem(nam);
-#elif defined(AFS_OBSD44_ENV)
-    code = sobind(newSocket, nam, osi_curproc());
 #else
     code = sobind(newSocket, nam);
 #endif
@@ -1096,12 +1070,8 @@ afs_rxevent_daemon(void)
 	if (afs_termState == AFSOP_STOP_RXEVENT) {
 #ifdef RXK_LISTENER_ENV
 	    afs_termState = AFSOP_STOP_RXK_LISTENER;
-#else
-#ifdef AFS_SUN510_ENV
-	    afs_termState = AFSOP_STOP_NETIF;
 #else
 	    afs_termState = AFSOP_STOP_COMPLETE;
-#endif
 #endif
 	    osi_rxWakeup(&afs_termState);
 	    return;
@@ -1169,7 +1139,9 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
     if (!code) {
 	p->length = nbytes - RX_HEADER_SIZE;;
 	if ((nbytes > tlen) || (p->length & 0x8000)) {	/* Bogus packet */
-	    if (nbytes <= 0) {
+	    if (nbytes > 0)
+		rxi_MorePackets(rx_initSendWindow);
+	    else {
 		MUTEX_ENTER(&rx_stats_mutex);
 		rx_stats.bogusPacketOnRead++;
 		rx_stats.bogusHost = from.sin_addr.s_addr;
@@ -1214,12 +1186,28 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
  * OS's socket receive routine returns as a result of a signal.
  */
 int rxk_ListenerPid;		/* Used to signal process to wakeup at shutdown */
-#ifdef AFS_LINUX20_ENV
-struct task_struct *rxk_ListenerTask;
-#endif
 
+#ifdef AFS_SUN5_ENV
+/*
+ * Run the listener as a kernel thread.
+ */
 void
 rxk_Listener(void)
+{
+    extern id_t syscid;
+    void rxk_ListenerProc(void);
+    if (thread_create
+	(NULL, DEFAULTSTKSZ, rxk_ListenerProc, 0, 0, &p0, TS_RUN,
+	 minclsyspri) == NULL)
+	osi_Panic("rxk_Listener: failed to start listener thread!\n");
+}
+
+void
+rxk_ListenerProc(void)
+#else /* AFS_SUN5_ENV */
+void
+rxk_Listener(void)
+#endif				/* AFS_SUN5_ENV */
 {
     struct rx_packet *rxp = NULL;
     int code;
@@ -1227,7 +1215,6 @@ rxk_Listener(void)
 
 #ifdef AFS_LINUX20_ENV
     rxk_ListenerPid = current->pid;
-    rxk_ListenerTask = current;
 #endif
 #ifdef AFS_SUN5_ENV
     rxk_ListenerPid = 1;	/* No PID, just a flag that we're alive */
@@ -1240,9 +1227,9 @@ rxk_Listener(void)
 #elif defined(AFS_DARWIN_ENV)
     rxk_ListenerPid = current_proc()->p_pid;
 #endif
-#ifdef RX_ENABLE_LOCKS
+#if defined(RX_ENABLE_LOCKS) && !defined(AFS_SUN5_ENV)
     AFS_GUNLOCK();
-#endif /* RX_ENABLE_LOCKS */
+#endif /* RX_ENABLE_LOCKS && !AFS_SUN5_ENV */
     while (afs_termState != AFSOP_STOP_RXK_LISTENER) {
 	if (rxp) {
 	    rxi_RestoreDataBufs(rxp);
@@ -1260,21 +1247,16 @@ rxk_Listener(void)
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
     if (afs_termState == AFSOP_STOP_RXK_LISTENER) {
-#ifdef AFS_SUN510_ENV
-	afs_termState = AFSOP_STOP_NETIF;
-#else
 	afs_termState = AFSOP_STOP_COMPLETE;
-#endif
 	osi_rxWakeup(&afs_termState);
     }
     rxk_ListenerPid = 0;
-#ifdef AFS_LINUX20_ENV
-    rxk_ListenerTask = 0;
-    osi_rxWakeup(&rxk_ListenerTask);
-#endif
-#if defined(AFS_SUN5_ENV)
+#if defined(AFS_LINUX22_ENV) || defined(AFS_SUN5_ENV)
     osi_rxWakeup(&rxk_ListenerPid);
 #endif
+#ifdef AFS_SUN5_ENV
+    AFS_GUNLOCK();
+#endif /* AFS_SUN5_ENV */
 }
 
 #if !defined(AFS_LINUX20_ENV) && !defined(AFS_SUN5_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_XBSD_ENV)
