@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxdebug/rxdebug.c,v 1.5.2.4 2008/09/27 20:54:55 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/rxdebug/rxdebug.c,v 1.6 2004/12/07 05:07:06 shadow Exp $");
 
 #include <sys/types.h>
 #include <errno.h>
@@ -32,7 +32,13 @@ RCSID
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <sys/stat.h>
 #include <afs/stds.h>
 #include <afs/cmd.h>
@@ -79,7 +85,9 @@ PortName(aname)
 }
 
 int
-MainCommand(struct cmd_syndesc *as, void *arock)
+MainCommand(as, arock)
+     char *arock;
+     struct cmd_syndesc *as;
 {
     register int i;
     int s;
@@ -270,7 +278,7 @@ MainCommand(struct cmd_syndesc *as, void *arock)
 	    fprintf(stderr,
 		    "WARNING: Server doesn't support retrieval of Rx statistics\n");
 	} else {
-	    struct rx_statistics rxstats;
+	    struct rx_stats rxstats;
 
 	    /* should gracefully handle the case where rx_stats grows */
 	    code =
@@ -584,7 +592,7 @@ main(argc, argv)
     }
 #endif
 
-    ts = cmd_CreateSyntax(NULL, MainCommand, NULL, "probe RX server");
+    ts = cmd_CreateSyntax(NULL, MainCommand, 0, "probe RX server");
     cmd_AddParm(ts, "-servers", CMD_SINGLE, CMD_REQUIRED, "server machine");
     cmd_AddParm(ts, "-port", CMD_SINGLE, CMD_OPTIONAL, "IP port");
     cmd_AddParm(ts, "-nodally", CMD_FLAG, CMD_OPTIONAL,
