@@ -20,7 +20,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxkad/domestic/fcrypt.c,v 1.11 2003/07/15 23:16:44 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rxkad/domestic/fcrypt.c,v 1.16 2005/06/02 05:20:23 shadow Exp $");
 
 #define DEBUG 0
 #ifdef KERNEL
@@ -55,7 +55,7 @@ RCSID
 #include "sboxes.h"
 #include "fcrypt.h"
 #include "rxkad.h"
-
+#include <des/stats.h>
 
 #ifdef TCRYPT
 int ROUNDS = 16;
@@ -102,19 +102,19 @@ fc_keysched(struct ktc_encryptionKey *key, fc_KeySchedule schedule)
 	kword[1] = (kword[1] >> 11) | (temp << (56 - 32 - 11));
 	schedule[i] = kword[0];
     }
-    LOCK_RXKAD_STATS rxkad_stats.fc_key_scheds++;
-    UNLOCK_RXKAD_STATS return 0;
+    INC_RXKAD_STATS(fc_key_scheds);
+    return 0;
 }
 
 /* IN int encrypt; * 0 ==> decrypt, else encrypt */
 afs_int32
-fc_ecb_encrypt(afs_uint32 * clear, afs_uint32 * cipher,
+fc_ecb_encrypt(void * clear, void * cipher,
 	       fc_KeySchedule schedule, int encrypt)
 {
     afs_uint32 L, R;
-    afs_uint32 S, P;
-    unsigned char *Pchar = (unsigned char *)&P;
-    unsigned char *Schar = (unsigned char *)&S;
+    volatile afs_uint32 S, P;
+    volatile unsigned char *Pchar = (unsigned char *)&P;
+    volatile unsigned char *Schar = (unsigned char *)&S;
     int i;
 
 #if defined(vax) || (defined(mips) && defined(MIPSEL)) || defined(AFSLITTLE_ENDIAN)
@@ -133,13 +133,13 @@ fc_ecb_encrypt(afs_uint32 * clear, afs_uint32 * cipher,
     memcpy(&L, clear, sizeof(afs_int32));
     memcpy(&R, clear + 1, sizeof(afs_int32));
 #else
-    L = ntohl(*clear);
-    R = ntohl(*(clear + 1));
+    L = ntohl(*((afs_uint32 *)clear));
+    R = ntohl(*((afs_uint32 *)clear + 1));
 #endif
 
     if (encrypt) {
-	LOCK_RXKAD_STATS rxkad_stats.fc_encrypts[ENCRYPT]++;
-	UNLOCK_RXKAD_STATS for (i = 0; i < (ROUNDS / 2); i++) {
+	INC_RXKAD_STATS(fc_encrypts[ENCRYPT]);
+	for (i = 0; i < (ROUNDS / 2); i++) {
 	    S = *schedule++ ^ R;	/* xor R with key bits from schedule */
 	    Pchar[Byte2] = sbox0[Schar[Byte0]];	/* do 8-bit S Box subst. */
 	    Pchar[Byte3] = sbox1[Schar[Byte1]];	/* and permute the result */
@@ -156,8 +156,8 @@ fc_ecb_encrypt(afs_uint32 * clear, afs_uint32 * cipher,
 	    R ^= P;
 	}
     } else {
-	LOCK_RXKAD_STATS rxkad_stats.fc_encrypts[DECRYPT]++;
-	UNLOCK_RXKAD_STATS schedule = &schedule[ROUNDS - 1];	/* start at end of key schedule */
+	INC_RXKAD_STATS(fc_encrypts[DECRYPT]);
+	schedule = &schedule[ROUNDS - 1];	/* start at end of key schedule */
 	for (i = 0; i < (ROUNDS / 2); i++) {
 	    S = *schedule-- ^ L;	/* xor R with key bits from schedule */
 	    Pchar[Byte2] = sbox0[Schar[Byte0]];	/* do 8-bit S Box subst. and */
@@ -179,8 +179,8 @@ fc_ecb_encrypt(afs_uint32 * clear, afs_uint32 * cipher,
     memcpy(cipher, &L, sizeof(afs_int32));
     memcpy(cipher + 1, &R, sizeof(afs_int32));
 #else
-    *cipher = htonl(L);
-    *(cipher + 1) = htonl(R);
+    *((afs_int32 *)cipher) = htonl(L);
+    *((afs_int32 *)cipher + 1) = htonl(R);
 #endif
     return 0;
 }
@@ -197,7 +197,7 @@ fc_ecb_encrypt(afs_uint32 * clear, afs_uint32 * cipher,
   afs_uint32 *xor; * 8 bytes of initialization vector *
 */
 afs_int32
-fc_cbc_encrypt(char *input, char *output, afs_int32 length,
+fc_cbc_encrypt(void *input, void *output, afs_int32 length,
 	       fc_KeySchedule key, afs_uint32 * xor, int encrypt)
 {
     afs_uint32 i, j;
@@ -209,7 +209,7 @@ fc_cbc_encrypt(char *input, char *output, afs_int32 length,
 	for (i = 0; length > 0; i++, length -= 8) {
 	    /* get input */
 	    memcpy(t_input, input, sizeof(t_input));
-	    input += sizeof(t_input);
+	    input=((char *)input) + sizeof(t_input);
 
 	    /* zero pad */
 	    for (j = length; j <= 7; j++)
@@ -223,7 +223,7 @@ fc_cbc_encrypt(char *input, char *output, afs_int32 length,
 
 	    /* copy temp output and save it for cbc */
 	    memcpy(output, t_output, sizeof(t_output));
-	    output += sizeof(t_output);
+	    output=(char *)output + sizeof(t_output);
 
 	    /* calculate xor value for next round from plain & cipher text */
 	    xor[0] = t_input[0] ^ t_output[0];
@@ -238,7 +238,7 @@ fc_cbc_encrypt(char *input, char *output, afs_int32 length,
 	for (i = 0; length > 0; i++, length -= 8) {
 	    /* get input */
 	    memcpy(t_input, input, sizeof(t_input));
-	    input += sizeof(t_input);
+	    input=((char *)input) + sizeof(t_input);
 
 	    /* no padding for decrypt */
 	    fc_ecb_encrypt(t_input, t_output, key, encrypt);
@@ -249,7 +249,7 @@ fc_cbc_encrypt(char *input, char *output, afs_int32 length,
 
 	    /* copy temp output */
 	    memcpy(output, t_output, sizeof(t_output));
-	    output += sizeof(t_output);
+	    output=((char *)output) + sizeof(t_output);
 
 	    /* calculate xor value for next round from plain & cipher text */
 	    xor[0] = t_input[0] ^ t_output[0];
