@@ -15,7 +15,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxkad/rxkad_server.c,v 1.13 2003/07/15 23:16:42 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rxkad/rxkad_server.c,v 1.17 2005/11/06 09:29:44 jaltman Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -36,6 +36,7 @@ RCSID
 #include <rx/xdr.h>
 #include <des.h>
 #include <afs/afsutil.h>
+#include <des/stats.h>
 #include "private_data.h"
 #define XPRT_RXKAD_SERVER
 
@@ -59,6 +60,9 @@ static struct rx_securityOps rxkad_server_ops = {
     rxkad_CheckPacket,		/* check data packet */
     rxkad_DestroyConnection,
     rxkad_GetStats,
+    0,				/* spare 1 */
+    0,				/* spare 2 */
+    0,				/* spare 3 */
 };
 extern afs_uint32 rx_MyMaxSendSize;
 
@@ -76,8 +80,8 @@ static fc_KeySchedule random_int32_schedule;
 
 #include <assert.h>
 pthread_mutex_t rxkad_random_mutex;
-#define LOCK_RM assert(pthread_mutex_lock(&rxkad_random_mutex)==0);
-#define UNLOCK_RM assert(pthread_mutex_unlock(&rxkad_random_mutex)==0);
+#define LOCK_RM assert(pthread_mutex_lock(&rxkad_random_mutex)==0)
+#define UNLOCK_RM assert(pthread_mutex_unlock(&rxkad_random_mutex)==0)
 #else
 #define LOCK_RM
 #define UNLOCK_RM
@@ -89,8 +93,10 @@ init_random_int32(void)
     struct timeval key;
 
     gettimeofday(&key, NULL);
-    LOCK_RM fc_keysched(&key, random_int32_schedule);
-UNLOCK_RM}
+    LOCK_RM;
+    fc_keysched((struct ktc_encryptionKey*)&key, random_int32_schedule);
+    UNLOCK_RM;
+}
 
 static afs_int32
 get_random_int32(void)
@@ -98,9 +104,11 @@ get_random_int32(void)
     static struct timeval seed;
     afs_int32 rc;
 
-    LOCK_RM fc_ecb_encrypt(&seed, &seed, random_int32_schedule, ENCRYPT);
+    LOCK_RM;
+    fc_ecb_encrypt(&seed, &seed, random_int32_schedule, ENCRYPT);
     rc = seed.tv_sec;
-    UNLOCK_RM return rc;
+    UNLOCK_RM;
+    return rc;
 }
 
 /* Called with four parameters.  The first is the level of encryption, as
@@ -153,8 +161,8 @@ rxkad_NewServerSecurityObject(rxkad_level level, char *get_key_rock,
     tsp->user_ok = user_ok;	/* to inform server of client id. */
     init_random_int32();
 
-    LOCK_RXKAD_STATS rxkad_stats_serverObjects++;
-    UNLOCK_RXKAD_STATS return tsc;
+    INC_RXKAD_STATS(serverObjects);
+    return tsc;
 }
 
 /* server: called to tell if a connection authenticated properly */
@@ -229,8 +237,8 @@ rxkad_GetChallenge(struct rx_securityClass *aobj, struct rx_connection *aconn,
     rx_packetwrite(apacket, 0, challengeSize, challenge);
     rx_SetDataSize(apacket, challengeSize);
     sconn->tried = 1;
-    LOCK_RXKAD_STATS rxkad_stats.challengesSent++;
-    UNLOCK_RXKAD_STATS return 0;
+    INC_RXKAD_STATS(challengesSent);
+    return 0;
 }
 
 /* server: process a response to a challenge packet */
@@ -394,11 +402,9 @@ rxkad_CheckResponse(struct rx_securityClass *aobj,
 	return RXKADLEVELFAIL;
     sconn->level = level;
     rxkad_SetLevel(aconn, sconn->level);
-    LOCK_RXKAD_STATS rxkad_stats.responses[rxkad_LevelIndex(sconn->level)]++;
-    UNLOCK_RXKAD_STATS
-	/* now compute endpoint-specific info used for computing 16 bit checksum */
-	rxkad_DeriveXORInfo(aconn, sconn->keysched, sconn->ivec,
-			    sconn->preSeq);
+    INC_RXKAD_STATS(responses[rxkad_LevelIndex(sconn->level)]);
+    /* now compute endpoint-specific info used for computing 16 bit checksum */
+    rxkad_DeriveXORInfo(aconn, sconn->keysched, sconn->ivec, sconn->preSeq);
 
     /* otherwise things are ok */
     sconn->expirationTime = end;
