@@ -11,12 +11,13 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/sys/afssyscalls.c,v 1.7 2003/07/15 23:16:54 shadow Exp $");
+    ("$Header: /cvs/openafs/src/sys/afssyscalls.c,v 1.12 2005/10/15 15:19:38 shadow Exp $");
 
 #include <signal.h>
 #include <sys/errno.h>
 #include <afs/afs_args.h>
 #include <sys/file.h>
+#include <sys/ioctl.h>
 #if defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
 #include <unistd.h>
 #else
@@ -58,7 +59,6 @@ static void check_iops(int index, char *fun, char *file, int line);
 #pragma weak xiinc = iinc
 #pragma weak xidec = idec
 #pragma weak xiopen = iopen
-#pragma weak xlsetpag = lsetpag
 #pragma weak xlpioctl = lpioctl
 #ifdef notdef
 #pragma weak xiread = iread
@@ -204,12 +204,6 @@ iwrite(int dev, int inode, int inode_p1, unsigned int offset, char *cbuf,
 }
 #endif /* notdef */
 
-int
-lsetpag(void)
-{
-    return (syscall(AFS_SETPAG));
-}
-
 int
 lpioctl(char *path, int cmd, char *cmarg, int follow)
 {
@@ -313,21 +307,50 @@ iwrite(int dev, int inode, int inode_p1, unsigned int offset, char *cbuf,
 
 #endif /* AFS_NAMEI_ENV */
 
-int
-lsetpag(void)
-{
-    int errcode;
+#if defined(AFS_DARWIN80_ENV)
+int ioctl_afs_syscall(long syscall, long param1, long param2, long param3, 
+		     long param4, long param5, long param6, int *rval) {
+  struct afssysargs syscall_data;
+  int code;
+  int fd = open(SYSCALL_DEV_FNAME, O_RDWR);
+  if(fd < 0)
+    return -1;
 
-    errcode = syscall(AFS_SYSCALL, AFSCALL_SETPAG);
-    return (errcode);
+  syscall_data.syscall = syscall;
+  syscall_data.param1 = param1;
+  syscall_data.param2 = param2;
+  syscall_data.param3 = param3;
+  syscall_data.param4 = param4;
+  syscall_data.param5 = param5;
+  syscall_data.param6 = param6;
+
+  code = ioctl(fd, VIOC_SYSCALL, &syscall_data);
+
+  close(fd);
+  if (code)
+     return code;
+  *rval=syscall_data.retval;
+  return 0;
 }
+#endif
 
 int
 lpioctl(char *path, int cmd, char *cmarg, int follow)
 {
-    int errcode;
+    int errcode, rval;
 
+#if defined(AFS_LINUX20_ENV)
+    rval = proc_afs_syscall(AFSCALL_PIOCTL, (long)path, cmd, (long)cmarg, follow, &errcode);
+
+    if(rval)
     errcode = syscall(AFS_SYSCALL, AFSCALL_PIOCTL, path, cmd, cmarg, follow);
+#elif defined(AFS_DARWIN80_ENV)
+    if (ioctl_afs_syscall(AFSCALL_PIOCTL,(long)path,cmd,(long)cmarg,follow,0,0,&errcode))
+        errcode=ENOSYS;
+#else
+    errcode = syscall(AFS_SYSCALL, AFSCALL_PIOCTL, path, cmd, cmarg, follow);
+#endif
+
     return (errcode);
 }
 
