@@ -19,11 +19,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.69 2006/02/01 16:09:19 shadow Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <string.h>
 #include <sys/types.h>
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 #include <sys/stat.h>
@@ -38,6 +39,14 @@
 #include <netdb.h>
 #include <unistd.h>		/* sysconf() */
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
+
 #ifndef ITIMER_REAL
 #include <sys/time.h>
 #endif /* ITIMER_REAL */
@@ -104,10 +113,10 @@ extern int BreakVolumeCallBacksLater();
 extern int LogLevel, etext;
 extern afs_int32 BlocksSpare, PctSpare;
 
-void *ShutDown(void *);
+void ShutDown(void);
 static void ClearXStatValues(), NewParms(), PrintCounters();
 static void ResetCheckDescriptors(void), ResetCheckSignal(void);
-static void *CheckSignal(void *);
+static void CheckSignal(void);
 extern int GetKeysFromToken();
 extern int RXAFS_ExecuteRequest();
 extern int RXSTATS_ExecuteRequest();
@@ -141,6 +150,8 @@ struct afsconf_dir *confDir;	/* Configuration dir object */
 
 int restartMode = RESTART_ORDINARY;
 
+int Testing = 0;		/* for ListViceInodes */
+
 /*
  * Home for the performance statistics.
  */
@@ -155,10 +166,7 @@ int SawSpare;
 int SawPctSpare;
 int debuglevel = 0;
 int printBanner = 0;
-int rxJumbograms = 0;		/* default is to not send and receive jumbograms. */
-int rxBind = 0;		/* don't bind */
-int rxkadDisableDotCheck = 0;      /* disable check for dot in principal name */ 
-int rxMaxMTU = -1;
+int rxJumbograms = 1;		/* default is to send and receive jumbograms. */
 afs_int32 implicitAdminRights = PRSFS_LOOKUP;	/* The ADMINISTER right is 
 						 * already implied */
 afs_int32 readonlyServer = 0;
@@ -173,12 +181,6 @@ int SawLock;
 #endif
 time_t StartTime;
 
-/**
- * seconds to wait until forcing a panic during ShutDownAndCore(PANIC)
- * in case we get stuck.
- */
-static int panic_timeout = 30 * 60;
-
 int rxpackets = 150;		/* 100 */
 int nSmallVns = 400;		/* 200 */
 int large = 400;		/* 200 */
@@ -194,10 +196,6 @@ int sendBufSize = 16384;	/* send buffer size */
 
 struct timeval tp;
 
-#ifdef AFS_PTHREAD_ENV
-pthread_key_t viced_uclient_key;
-#endif
-
 /*
  * FileServer's name and IP address, both network byte order and
  * host byte order.
@@ -218,8 +216,8 @@ static void FlagMsg();
  */
 
 /* DEBUG HACK */
-static void *
-CheckDescriptors(void *unused)
+static void
+CheckDescriptors()
 {
 #ifndef AFS_NT40_ENV
     struct afs_stat status;
@@ -235,7 +233,6 @@ CheckDescriptors(void *unused)
     fflush(stdout);
     ResetCheckDescriptors();
 #endif
-    return 0;
 }				/*CheckDescriptors */
 
 
@@ -243,19 +240,19 @@ CheckDescriptors(void *unused)
 void
 CheckSignal_Signal(x)
 {
-    CheckSignal(NULL);
+    CheckSignal();
 }
 
 void
 ShutDown_Signal(x)
 {
-    ShutDown(NULL);
+    ShutDown();
 }
 
 void
 CheckDescriptors_Signal(x)
 {
-    CheckDescriptors(NULL);
+    CheckDescriptors();
 }
 #else /* AFS_PTHREAD_ENV */
 void
@@ -319,16 +316,16 @@ ResetCheckDescriptors(void)
 }
 
 #if defined(AFS_PTHREAD_ENV)
-int
+char *
 threadNum(void)
 {
-    return (int)pthread_getspecific(rx_thread_id_key);
+    return pthread_getspecific(rx_thread_id_key);
 }
 #endif
 
 /* proc called by rxkad module to get a key */
 static int
-get_key(void *arock, register afs_int32 akvno, char *akey)
+get_key(char *arock, register afs_int32 akvno, char *akey)
 {
     /* find the key */
     static struct afsconf_key tkey;
@@ -339,10 +336,8 @@ get_key(void *arock, register afs_int32 akvno, char *akey)
 	return 1;
     }
     code = afsconf_GetKey(confDir, akvno, tkey.key);
-    if (code) {
-	ViceLog(0, ("afsconf_GetKey failure: kvno %d code %d\n", akvno, code));
+    if (code)
 	return code;
-    }
     memcpy(akey, tkey.key, sizeof(tkey.key));
     return 0;
 
@@ -393,6 +388,7 @@ CheckAdminName()
 
 }				/*CheckAdminName */
 
+
 static void
 setThreadId(char *s)
 {
@@ -409,8 +405,8 @@ setThreadId(char *s)
 }
 
 /* This LWP does things roughly every 5 minutes */
-static void *
-FiveMinuteCheckLWP(void *unused)
+static void
+FiveMinuteCheckLWP()
 {
     static int msg = 0;
     char tbuffer[32];
@@ -450,7 +446,6 @@ FiveMinuteCheckLWP(void *unused)
 	    }
 	}
     }
-    return NULL;
 }				/*FiveMinuteCheckLWP */
 
 
@@ -458,8 +453,8 @@ FiveMinuteCheckLWP(void *unused)
  * other 5 minute activities because it may be delayed by timeouts when
  * it probes the workstations
  */
-static void *
-HostCheckLWP(void *unused)
+static void
+HostCheckLWP()
 {
     ViceLog(1, ("Starting Host check process\n"));
     setThreadId("HostCheckLWP");
@@ -472,15 +467,14 @@ HostCheckLWP(void *unused)
 	ViceLog(2, ("Checking for dead venii & clients\n"));
 	h_CheckHosts();
     }
-    return NULL;
 }				/*HostCheckLWP */
 
 /* This LWP does fsync checks every 5 minutes:  it should not be used for
  * other 5 minute activities because it may be delayed by timeouts when
  * it probes the workstations
  */
-static void *
-FsyncCheckLWP(void *unused)
+static
+FsyncCheckLWP()
 {
     afs_int32 code;
 #ifdef AFS_PTHREAD_ENV
@@ -490,6 +484,11 @@ FsyncCheckLWP(void *unused)
 
     setThreadId("FsyncCheckLWP");
 
+#ifdef AFS_PTHREAD_ENV
+    assert(pthread_cond_init(&fsync_cond, NULL) == 0);
+    assert(pthread_mutex_init(&fsync_glock_mutex, NULL) == 0);
+#endif
+
     while (1) {
 	FSYNC_LOCK;
 #ifdef AFS_PTHREAD_ENV
@@ -512,7 +511,6 @@ FsyncCheckLWP(void *unused)
 	    code = BreakLaterCallBacks();
 	} while (code != 0);
     }
-    return NULL;
 }
 
 /*------------------------------------------------------------------------
@@ -627,8 +625,8 @@ PrintCounters()
 
 
 
-static void *
-CheckSignal(void *unused)
+static void
+CheckSignal()
 {
     if (FS_registered > 0) {
 	/*
@@ -642,18 +640,8 @@ CheckSignal(void *unused)
     DumpCallBackState();
     PrintCounters();
     ResetCheckSignal();
-    return NULL;
-}				/*CheckSignal */
 
-static void *
-ShutdownWatchdogLWP(void *unused)
-{
-    sleep(panic_timeout);
-    ViceLog(0, ("ShutdownWatchdogLWP: Failed to shutdown and panic "
-                "within %d seconds; forcing panic\n", panic_timeout));
-    assert(0);
-    return NULL;
-}
+}				/*CheckSignal */
 
 void
 ShutDownAndCore(int dopanic)
@@ -661,20 +649,6 @@ ShutDownAndCore(int dopanic)
     time_t now = time(0);
     char tbuffer[32];
 
-    if (dopanic) {
-#ifdef AFS_PTHREAD_ENV
-	pthread_t watchdogPid;
-	pthread_attr_t tattr;
-	assert(pthread_attr_init(&tattr) == 0);
-	assert(pthread_create(&watchdogPid, &tattr, ShutdownWatchdogLWP, NULL) == 0);
-#else
-	PROCESS watchdogPid;
-	assert(LWP_CreateProcess
-	       (ShutdownWatchdogLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	        NULL, "ShutdownWatchdog", &watchdogPid) == LWP_SUCCESS);
-#endif
-    }
-
     ViceLog(0,
 	    ("Shutting down file server at %s",
 	     afs_ctime(&now, tbuffer, sizeof(tbuffer))));
@@ -720,18 +694,14 @@ ShutDownAndCore(int dopanic)
 	}
     }
 
-    if (dopanic)
-	assert(0);
-
     exit(0);
 
 }				/*ShutDown */
 
-void *
-ShutDown(void *unused)
+void
+ShutDown(void)
 {				/* backward compatibility */
     ShutDownAndCore(DONTPANIC);
-    return NULL;
 }
 
 
@@ -765,9 +735,6 @@ FlagMsg()
     strcat(buffer, "[-rxpck <number of rx extra packets>] ");
     strcat(buffer, "[-rxdbg (enable rx debugging)] ");
     strcat(buffer, "[-rxdbge (enable rxevent debugging)] ");
-    strcat(buffer, "[-rxmaxmtu <bytes>] ");
-    strcat(buffer, "[-rxbind (bind the Rx socket to one address)] ");
-    strcat(buffer, "[-allow-dotted-principals (disable the rxkad principal name dot check)] ");
 #if AFS_PTHREAD_ENV
     strcat(buffer, "[-vattachpar <number of volume attach threads>] ");
 #endif
@@ -784,8 +751,6 @@ FlagMsg()
     strcat(buffer, "[-udpsize <size of socket buffer in bytes>] ");
     strcat(buffer, "[-sendsize <size of send buffer in bytes>] ");
     strcat(buffer, "[-abortthreshold <abort threshold>] ");
-    strcat(buffer, "[-nojumbo (disable jumbogram network packets - deprecated)] ");
-    strcat(buffer, "[-jumbo (enable jumbogram network packets)] ");
 /*   strcat(buffer, "[-enable_peer_stats] "); */
 /*   strcat(buffer, "[-enable_process_stats] "); */
     strcat(buffer, "[-help]\n");
@@ -1086,25 +1051,6 @@ ParseArgs(int argc, char *argv[])
 #endif
 	else if (!strcmp(argv[i], "-nojumbo")) {
 	    rxJumbograms = 0;
-	} else if (!strcmp(argv[i], "-jumbo")) {
-	    rxJumbograms = 1;
-	} else if (!strcmp(argv[i], "-rxbind")) {
-	    rxBind = 1;
-	} else if (!strcmp(argv[i], "-allow-dotted-principals")) {
-	    rxkadDisableDotCheck = 1;
-	} else if (!strcmp(argv[i], "-rxmaxmtu")) {
-	    if ((i + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
-		return -1; 
-	    }
-	    rxMaxMTU = atoi(argv[++i]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
-		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
-		       rxMaxMTU, RX_MIN_PACKET_SIZE, 
-		       RX_MAX_PACKET_DATA_SIZE);
-		return -1;
-	    }
 	} else if (!strcmp(argv[i], "-realm")) {
 	    extern char local_realms[AFS_NUM_LREALMS][AFS_REALM_SZ];
 	    extern int  num_lrealms;
@@ -1157,9 +1103,34 @@ ParseArgs(int argc, char *argv[])
 	    rx_enableProcessRPCStats();
 	}
 	else if (strcmp(argv[i], "-auditlog") == 0) {
+	    int tempfd, flags;
+	    FILE *auditout;
+	    char oldName[MAXPATHLEN];
 	    char *fileName = argv[++i];
-
-            osi_audit_file(fileName);
+	    
+#ifndef AFS_NT40_ENV
+	    struct stat statbuf;
+	    
+	    if ((lstat(fileName, &statbuf) == 0) 
+		&& (S_ISFIFO(statbuf.st_mode))) {
+		flags = O_WRONLY | O_NONBLOCK;
+	    } else 
+#endif
+	    {
+		strcpy(oldName, fileName);
+		strcat(oldName, ".old");
+		renamefile(fileName, oldName);
+		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	    }
+	    tempfd = open(fileName, flags, 0666);
+	    if (tempfd > -1) {
+		auditout = fdopen(tempfd, "a");
+		if (auditout) {
+		    osi_audit_file(auditout);
+		} else
+		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	}
 #ifndef AFS_NT40_ENV
 	else if (strcmp(argv[i], "-syslog") == 0) {
@@ -1208,7 +1179,7 @@ ParseArgs(int argc, char *argv[])
 	if (!Sawcbs)
 	    numberofcbs = 64000;
 	if (!Sawlwps)
-	    lwps = 128;
+	    lwps = 12;
 	if (!Sawbufs)
 	    buffs = 120;
 	if (!SawVC)
@@ -1316,11 +1287,6 @@ InitPR()
 		("Couldn't initialize protection library; code=%d.\n", code));
 	return code;
     }
-
-#ifdef AFS_PTHREAD_ENV
-    assert(pthread_key_create(&viced_uclient_key, NULL) == 0);
-#endif
-
     SystemId = SYSADMINID;
     SystemAnyUser = ANYUSERID;
     SystemAnyUserCPS.prlist_len = 0;
@@ -1388,7 +1354,6 @@ vl_Initialize(char *confDir)
 			     info.hostAddr[i].sin_port, USER_SERVICE_ID, sc,
 			     scIndex);
     code = ubik_ClientInit(serverconns, &cstruct);
-    afsconf_Close(tdir);
     if (code) {
 	ViceLog(0, ("vl_Initialize: ubik client init failed.\n"));
 	return code;
@@ -1460,22 +1425,16 @@ ReadSysIdFile()
 		 AFSDIR_SERVER_SYSID_FILEPATH, nentries));
 	return EIO;
     }
-    if (FS_HostAddr_cnt == 0) {
-	FS_HostAddr_cnt = nentries;
-	for (i = 0; i < nentries; i++) {
-	    if (read(fd, (char *)&FS_HostAddrs[i], sizeof(afs_int32)) !=
-		sizeof(afs_int32)) {
-		ViceLog(0,
-			("%s: Read of addresses failed (%d)\n",
-			 AFSDIR_SERVER_SYSID_FILEPATH, errno));
-		FS_HostAddr_cnt = 0;	/* reset it */
-		return EIO;
-	    }
+    FS_HostAddr_cnt = nentries;
+    for (i = 0; i < nentries; i++) {
+	if (read(fd, (char *)&FS_HostAddrs[i], sizeof(afs_int32)) !=
+	    sizeof(afs_int32)) {
+	    ViceLog(0,
+		    ("%s: Read of addresses failed (%d)\n",
+		     AFSDIR_SERVER_SYSID_FILEPATH, errno));
+	    FS_HostAddr_cnt = 0;	/* reset it */
+	    return EIO;
 	}
-    } else {
-	ViceLog(1,
-		("%s: address list ignored (NetInfo/NetRestrict override)\n",
-		 AFSDIR_SERVER_SYSID_FILEPATH));
     }
     close(fd);
     return 0;
@@ -1563,7 +1522,7 @@ Do_VLRegisterRPC()
 	FS_HostAddrs_HBO[i] = ntohl(FS_HostAddrs[i]);
     addrs.bulkaddrs_len = FS_HostAddr_cnt;
     addrs.bulkaddrs_val = (afs_uint32 *) FS_HostAddrs_HBO;
-    code = ubik_VL_RegisterAddrs(cstruct, 0, &FS_HostUUID, 0, &addrs);
+    code = ubik_Call(VL_RegisterAddrs, cstruct, 0, &FS_HostUUID, 0, &addrs);
     if (code) {
 	if (code == VL_MULTIPADDR) {
 	    ViceLog(0,
@@ -1579,7 +1538,6 @@ Do_VLRegisterRPC()
 	    ViceLog(0,
 		    ("VL_RegisterAddrs rpc failed; will retry periodically (code=%d, err=%d)\n",
 		     code, errno));
-	    FS_registered = 1;	/* Retry in the gc daemon */
 	}
     } else {
 	FS_registered = 2;	/* So we don't have to retry in the gc daemon */
@@ -1589,43 +1547,6 @@ Do_VLRegisterRPC()
     return 0;
 }
 
-afs_int32
-SetupVL()
-{
-    afs_int32 code;
-    extern int rxi_numNetAddrs;
-    extern afs_uint32 rxi_NetAddrs[];
-
-#ifndef AFS_NT40_ENV
-    if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
-	/*
-	 * Find addresses we are supposed to register as per the netrestrict 
-	 * and netinfo files (/usr/afs/local/NetInfo and 
-	 * /usr/afs/local/NetRestict)
-	 */
-	char reason[1024];
-	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
-				       ADDRSPERSITE, reason,
-				       AFSDIR_SERVER_NETINFO_FILEPATH,
-				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-	if (code < 0) {
-	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
-	    exit(1);
-	}
-	FS_HostAddr_cnt = (afs_uint32) code;
-    } else
-#endif
-    {
-	FS_HostAddr_cnt = rx_getAllAddr(FS_HostAddrs, ADDRSPERSITE);
-    }
-
-    if (FS_HostAddr_cnt == 1 && rxBind == 1)
-	code = FS_HostAddrs[0];
-    else 
-	code = htonl(INADDR_ANY);
-    return code;
-}
-
 afs_int32
 InitVL()
 {
@@ -1659,7 +1580,30 @@ InitVL()
     /* A good sysid file exists; inform the vlserver. If any conflicts,
      * we always use the latest interface available as the real truth.
      */
+#ifndef AFS_NT40_ENV
+    if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
+	/*
+	 * Find addresses we are supposed to register as per the netrestrict 
+	 * and netinfo files (/usr/afs/local/NetInfo and 
+	 * /usr/afs/local/NetRestict)
+	 */
+	char reason[1024];
+	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
+				       ADDRSPERSITE, reason,
+				       AFSDIR_SERVER_NETINFO_FILEPATH,
+				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	if (code < 0) {
+	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
+	    exit(1);
+	}
+	FS_HostAddr_cnt = (afs_uint32) code;
+    } else
+#endif
+    {
+	FS_HostAddr_cnt = rx_getAllAddr(FS_HostAddrs, ADDRSPERSITE);
+    }
 
+    FS_registered = 1;
     code = Do_VLRegisterRPC();
     return code;
 }
@@ -1684,7 +1628,6 @@ main(int argc, char *argv[])
 #endif
     int curLimit;
     time_t t;
-    afs_uint32 rx_bindhost;
 
 #ifdef	AFS_AIX32_ENV
     struct sigaction nsa;
@@ -1840,23 +1783,17 @@ main(int argc, char *argv[])
     rx_SetBusyThreshold(busy_threshold, VBUSY);
     rx_SetCallAbortThreshold(abort_threshold);
     rx_SetConnAbortThreshold(abort_threshold);
-#ifdef AFS_XBSD_ENV
-    stackSize = 128 * 1024;
-#else
     stackSize = lwps * 4000;
     if (stackSize < 32000)
 	stackSize = 32000;
     else if (stackSize > 44000)
 	stackSize = 44000;
-#endif
-#if defined(AFS_HPUX_ENV) || defined(AFS_SUN_ENV) || defined(AFS_SGI51_ENV) || defined(AFS_XBSD_ENV)
+#if    defined(AFS_HPUX_ENV) || defined(AFS_SUN_ENV) || defined(AFS_SGI51_ENV)
     rx_SetStackSize(1, stackSize);
 #endif
     if (udpBufSize)
 	rx_SetUdpBufSize(udpBufSize);	/* set the UDP buffer size for receive */
-    rx_bindhost = SetupVL();
-
-    if (rx_InitHost(rx_bindhost, (int)htons(7000)) < 0) {
+    if (rx_Init((int)htons(7000)) < 0) {
 	ViceLog(0, ("Cannot initialize RX\n"));
 	exit(1);
     }
@@ -1864,41 +1801,26 @@ main(int argc, char *argv[])
 	/* Don't send and don't allow 3.4 clients to send jumbograms. */
 	rx_SetNoJumbo();
     }
-    if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
-    }
     rx_GetIFInfo();
     rx_SetRxDeadTime(30);
     sc[0] = rxnull_NewServerSecurityObject();
     sc[1] = 0;			/* rxvab_NewServerSecurityObject(key1, 0) */
     sc[2] = rxkad_NewServerSecurityObject(rxkad_clear, NULL, get_key, NULL);
     sc[3] = rxkad_NewServerSecurityObject(rxkad_crypt, NULL, get_key, NULL);
-#ifdef USE_RXKAD_KEYTAB
-    if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0) {
-        rxkad_BindKeytabDecrypt(sc[2]);
-        rxkad_BindKeytabDecrypt(sc[3]);
-    }
-#endif
-    tservice = rx_NewServiceHost(rx_bindhost,  /* port */ 0, /* service id */ 
-				 1,	/*service name */
-				 "AFS",
-				 /* security classes */ sc,
-				 /* numb sec classes */
-				 4, RXAFS_ExecuteRequest);
+    tservice = rx_NewService( /* port */ 0, /* service id */ 1,	/*service name */
+			     "AFS",
+							/* security classes */ sc,
+							/* numb sec classes */
+			     4, RXAFS_ExecuteRequest);
     if (!tservice) {
 	ViceLog(0,
 		("Failed to initialize RX, probably two servers running.\n"));
 	exit(-1);
     }
-    if (rxkadDisableDotCheck) {
-        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-                                    NULL);
-    }
+    rx_SetDestroyConnProc(tservice, (void (*)())h_FreeConnection);
     rx_SetMinProcs(tservice, 3);
     rx_SetMaxProcs(tservice, lwps);
     rx_SetCheckReach(tservice, 1);
-    rx_SetServerIdleDeadErr(tservice, VNOSERVICE);
 
     tservice =
 	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", sc, 4,
@@ -1932,13 +1854,13 @@ main(int argc, char *argv[])
     ClearXStatValues();
 
     code = InitVL();
-    if (code && code != VL_MULTIPADDR) {
+    if (code) {
 	ViceLog(0, ("Fatal error in library initialization, exiting!!\n"));
 	exit(1);
     }
 
     code = InitPR();
-    if (code && code != -1) {
+    if (code) {
 	ViceLog(0, ("Fatal error in protection initialization, exiting!!\n"));
 	exit(1);
     }
@@ -1946,11 +1868,6 @@ main(int argc, char *argv[])
     /* allow super users to manage RX statistics */
     rx_SetRxStatUserOk(fs_rxstat_userok);
 
-#ifdef AFS_PTHREAD_ENV
-    assert(pthread_cond_init(&fsync_cond, NULL) == 0);
-    assert(pthread_mutex_init(&fsync_glock_mutex, NULL) == 0);
-#endif
-
     rx_StartServer(0);		/* now start handling requests */
 
     /* we ensure that there is enough space in the vnode buffer to satisfy
@@ -1996,25 +1913,25 @@ main(int argc, char *argv[])
     assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
 
     assert(pthread_create
-	   (&serverPid, &tattr, FiveMinuteCheckLWP,
+	   (&serverPid, &tattr, (void *)FiveMinuteCheckLWP,
 	    &fiveminutes) == 0);
     assert(pthread_create
-	   (&serverPid, &tattr, HostCheckLWP, &fiveminutes) == 0);
+	   (&serverPid, &tattr, (void *)HostCheckLWP, &fiveminutes) == 0);
     assert(pthread_create
-	   (&serverPid, &tattr, FsyncCheckLWP, &fiveminutes) == 0);
+	   (&serverPid, &tattr, (void *)FsyncCheckLWP, &fiveminutes) == 0);
 #else /* AFS_PTHREAD_ENV */
     ViceLog(5, ("Starting LWP\n"));
     assert(LWP_CreateProcess
 	   (FiveMinuteCheckLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	    &fiveminutes, "FiveMinuteChecks",
+	    (void *)&fiveminutes, "FiveMinuteChecks",
 	    &serverPid) == LWP_SUCCESS);
 
     assert(LWP_CreateProcess
 	   (HostCheckLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	    &fiveminutes, "HostCheck", &serverPid) == LWP_SUCCESS);
+	    (void *)&fiveminutes, "HostCheck", &serverPid) == LWP_SUCCESS);
     assert(LWP_CreateProcess
 	   (FsyncCheckLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	    &fiveminutes, "FsyncCheck", &serverPid) == LWP_SUCCESS);
+	    (void *)&fiveminutes, "FsyncCheck", &serverPid) == LWP_SUCCESS);
 #endif /* AFS_PTHREAD_ENV */
 
     TM_GetTimeOfDay(&tp, 0);
@@ -2074,5 +1991,4 @@ main(int argc, char *argv[])
 #else /* AFS_PTHREAD_ENV */
     assert(LWP_WaitProcess(&parentPid) == LWP_SUCCESS);
 #endif /* AFS_PTHREAD_ENV */
-    return 0;
 }
