@@ -10,9 +10,10 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID
+    ("$Header: /cvs/openafs/src/volser/volmain.c,v 1.21 2006/02/01 16:09:20 shadow Exp $");
 
 #include <sys/types.h>
-#include <string.h>
 #ifdef AFS_NT40_ENV
 #include <time.h>
 #include <fcntl.h>
@@ -23,6 +24,13 @@
 #include <sys/file.h>
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <rx/xdr.h>
 #include <afs/afsint.h>
 #include <stdio.h>
@@ -65,7 +73,6 @@
 #define VolserVersion "2.0"
 #define N_SECURITY_OBJECTS 3
 
-extern struct Lock localLock;
 extern struct volser_trans *TransList();
 #ifndef AFS_PTHREAD_ENV
 extern int (*vol_PollProc) ();
@@ -83,16 +90,11 @@ extern void RXSTATS_ExecuteRequest();
 struct afsconf_dir *tdir;
 static afs_int32 runningCalls = 0;
 int DoLogging = 0;
-int debuglevel = 0; 
 #define MAXLWP 16
 int lwps = 9;
 int udpBufSize = 0;		/* UDP buffer size for receive */
 
-int rxBind = 0;
-int rxkadDisableDotCheck = 0;
-
-#define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
-afs_uint32 SHostAddrs[ADDRSPERSITE];
+int Testing = 0;		/* for ListViceInodes */
 
 #define VS_EXIT(code)  {                                          \
                           osi_audit(VS_ExitEvent, code, AUD_END); \
@@ -142,8 +144,8 @@ TryUnlock()
 }
 
 /* background daemon for timing out transactions */
-static void*
-BKGLoop(void *unused)
+static void
+BKGLoop()
 {
     struct timeval tv;
     int loop = 0;
@@ -164,14 +166,12 @@ BKGLoop(void *unused)
 	    ReOpenLog(AFSDIR_SERVER_VOLSERLOG_FILEPATH);
 	}
     }
-
-    return NULL;
 }
 
 /* Background daemon for sleeping so the volserver does not become I/O bound */
 afs_int32 TTsleep, TTrun;
-static void *
-BKGSleep(void *unused)
+static void
+BKGSleep()
 {
     struct volser_trans *tt;
 
@@ -184,17 +184,13 @@ BKGSleep(void *unused)
 #endif
 	    VTRANS_LOCK;
 	    for (tt = TransList(); tt; tt = tt->next) {
-                VTRANS_OBJ_LOCK(tt);
 		if ((strcmp(tt->lastProcName, "DeleteVolume") == 0)
 		    || (strcmp(tt->lastProcName, "Clone") == 0)
 		    || (strcmp(tt->lastProcName, "ReClone") == 0)
 		    || (strcmp(tt->lastProcName, "Forward") == 0)
 		    || (strcmp(tt->lastProcName, "Restore") == 0)
-		    || (strcmp(tt->lastProcName, "ForwardMulti") == 0)) {
-                    VTRANS_OBJ_UNLOCK(tt);
+		    || (strcmp(tt->lastProcName, "ForwardMulti") == 0))
 		    break;
-                }
-                VTRANS_OBJ_UNLOCK(tt);
 	    }
 	    if (tt) {
 	        VTRANS_UNLOCK;
@@ -203,7 +199,6 @@ BKGSleep(void *unused)
 	        VTRANS_UNLOCK;
 	}
     }
-    return NULL;
 }
 
 #ifndef AFS_NT40_ENV
@@ -246,10 +241,8 @@ main(int argc, char **argv)
     int rxpackets = 100;
     char commandLine[150];
     int i;
-    int rxJumbograms = 0;	/* default is to send and receive jumbograms. */
-    int rxMaxMTU = -1;
+    int rxJumbograms = 1;	/* default is to send and receive jumbograms. */
     int bufSize = 0;		/* temp variable to read in udp socket buf size */
-    afs_uint32 host = ntohl(INADDR_ANY);
 
 #ifdef	AFS_AIX32_ENV
     /*
@@ -294,17 +287,6 @@ main(int argc, char **argv)
 	    DoLogging = 1;
 	} else if (strcmp(argv[code], "-help") == 0) {
 	    goto usage;
-	} else if (strcmp(argv[code], "-rxbind") == 0) {
-	    rxBind = 1;
-	} else if (strcmp(argv[code], "-allow-dotted-principals") == 0) {
-	    rxkadDisableDotCheck = 1;
-	} else if (strcmp(argv[code], "-d") == 0) {
-	    if ((code + 1) >= argc) {
-		fprintf(stderr, "missing argument for -d\n"); 
-		return -1; 
-	    }
-	    debuglevel = atoi(argv[++code]);
-	    LogLevel = debuglevel;
 	} else if (strcmp(argv[code], "-p") == 0) {
 	    lwps = atoi(argv[++code]);
 	    if (lwps > MAXLWP) {
@@ -313,27 +295,37 @@ main(int argc, char **argv)
 		lwps = MAXLWP;
 	    }
 	} else if (strcmp(argv[code], "-auditlog") == 0) {
+	    int tempfd, flags;
+	    FILE *auditout;
+	    char oldName[MAXPATHLEN];
 	    char *fileName = argv[++code];
 
-            osi_audit_file(fileName);
-            osi_audit(VS_StartEvent, 0, AUD_END);
+#ifndef AFS_NT40_ENV
+	    struct stat statbuf;
+	    
+	    if ((lstat(fileName, &statbuf) == 0) 
+		&& (S_ISFIFO(statbuf.st_mode))) {
+		flags = O_WRONLY | O_NONBLOCK;
+	    } else 
+#endif
+	    {
+		strcpy(oldName, fileName);
+		strcat(oldName, ".old");
+		renamefile(fileName, oldName);
+		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	    }
+	    tempfd = open(fileName, flags, 0666);
+	    if (tempfd > -1) {
+		auditout = fdopen(tempfd, "a");
+		if (auditout) {
+		    osi_audit_file(auditout);
+		    osi_audit(VS_StartEvent, 0, AUD_END);
+		} else
+		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
+	    } else
+		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	} else if (strcmp(argv[code], "-nojumbo") == 0) {
 	    rxJumbograms = 0;
-	} else if (strcmp(argv[code], "-jumbo") == 0) {
-	    rxJumbograms = 1;
-	} else if (!strcmp(argv[code], "-rxmaxmtu")) {
-	    if ((code + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
-		exit(1); 
-	    }
-	    rxMaxMTU = atoi(argv[++code]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
-		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d% invalid; must be between %d-%d\n",
-		       rxMaxMTU, RX_MIN_PACKET_SIZE, 
-		       RX_MAX_PACKET_DATA_SIZE);
-		exit(1);
-	    }
 	} else if (strcmp(argv[code], "-sleep") == 0) {
 	    sscanf(argv[++code], "%d/%d", &TTsleep, &TTrun);
 	    if ((TTsleep < 0) || (TTrun <= 0)) {
@@ -372,16 +364,14 @@ main(int argc, char **argv)
 	  usage:
 #ifndef AFS_NT40_ENV
 	    printf("Usage: volserver [-log] [-p <number of processes>] "
-		   "[-auditlog <log path>] [-d <debug level>] "
-		   "[-nojumbo] [-jumbo] [-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
+		   "[-auditlog <log path>] "
 		   "[-udpsize <size of socket buffer in bytes>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
 #else
 	    printf("Usage: volserver [-log] [-p <number of processes>] "
-		   "[-auditlog <log path>] [-d <debug level>] "
-		   "[-nojumbo] [-jumbo] [-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
+		   "[-auditlog <log path>] "
 		   "[-udpsize <size of socket buffer in bytes>] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
@@ -409,13 +399,7 @@ main(int argc, char **argv)
 	exit(1);
     }
 #endif
-    /* Open VolserLog and map stdout, stderr into it; VInitVolumePackage can
-       log, so we need to do this here */
-    OpenLog(AFSDIR_SERVER_VOLSERLOG_FILEPATH);
-
     VInitVolumePackage(volumeUtility, 0, 0, CONNECT_FS, 0);
-    /* For nuke() */
-    Lock_Init(&localLock);
     DInit(40);
 #ifndef AFS_PTHREAD_ENV
     vol_PollProc = IOMGR_Poll;	/* tell vol pkg to poll io system periodically */
@@ -426,26 +410,7 @@ main(int argc, char **argv)
     rx_nPackets = rxpackets;	/* set the max number of packets */
     if (udpBufSize)
 	rx_SetUdpBufSize(udpBufSize);	/* set the UDP buffer size for receive */
-    if (rxBind) {
-	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
-        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
-            AFSDIR_SERVER_NETINFO_FILEPATH) {
-            char reason[1024];
-            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-                                           ADDRSPERSITE, reason,
-                                           AFSDIR_SERVER_NETINFO_FILEPATH,
-                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-        } else 
-#endif	
-	{
-            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
-        }
-        if (ccode == 1) 
-            host = SHostAddrs[0];
-    }
-
-    code = rx_InitHost(host, (int)htons(AFSCONF_VOLUMEPORT));
+    code = rx_Init((int)htons(AFSCONF_VOLUMEPORT));
     if (code) {
 	fprintf(stderr, "rx init failed on socket AFSCONF_VOLUMEPORT %u\n",
 		AFSCONF_VOLUMEPORT);
@@ -455,13 +420,12 @@ main(int argc, char **argv)
 	/* Don't allow 3.4 vos clients to send jumbograms and we don't send. */
 	rx_SetNoJumbo();
     }
-    if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
-    }
     rx_GetIFInfo();
     rx_SetRxDeadTime(420);
     memset(busyFlags, 0, sizeof(busyFlags));
 
+    /* Open FileLog and map stdout, stderr into it */
+    OpenLog(AFSDIR_SERVER_VOLSERLOG_FILEPATH);
     SetupLogSignals();
 
     {
@@ -494,12 +458,8 @@ main(int argc, char **argv)
 	rxkad_NewServerSecurityObject(0, tdir, afsconf_GetKey, NULL);
     if (securityObjects[0] == (struct rx_securityClass *)0)
 	Abort("rxnull_NewServerSecurityObject");
-#ifdef USE_RXKAD_KEYTAB
-    if (securityObjects[2] != NULL && rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
-        rxkad_BindKeytabDecrypt(securityObjects[2]);
-#endif
     service =
-	rx_NewServiceHost(host, 0, VOLSERVICE_ID, "VOLSER", securityObjects, 3,
+	rx_NewService(0, VOLSERVICE_ID, "VOLSER", securityObjects, 3,
 		      AFSVolExecuteRequest);
     if (service == (struct rx_service *)0)
 	Abort("rx_NewService");
@@ -509,20 +469,12 @@ main(int argc, char **argv)
     if (lwps < 4)
 	lwps = 4;
     rx_SetMaxProcs(service, lwps);
-#if defined(AFS_XBSD_ENV)
-    rx_SetStackSize(service, (128 * 1024));
-#elif defined(AFS_SGI_ENV)
-    rx_SetStackSize(service, (48 * 1024));
+#ifdef AFS_SGI_ENV
+    rx_SetStackSize(service, 49152);
 #else
-    rx_SetStackSize(service, (32 * 1024));
+    rx_SetStackSize(service, 32768);
 #endif
 
-    if (rxkadDisableDotCheck) {
-        rx_SetSecurityConfiguration(service, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-                                    NULL);
-    }
-
     service =
 	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", securityObjects, 3,
 		      RXSTATS_ExecuteRequest);
@@ -545,5 +497,4 @@ main(int argc, char **argv)
 
     osi_audit(VS_FinishEvent, (-1), AUD_END);
     Abort("StartServer returned?");
-    return 0; /* not reached */
 }
