@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/volprocs.c,v 1.34.2.2 2004/11/09 17:13:45 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/volprocs.c,v 1.41 2006/01/25 03:24:25 shadow Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -1544,6 +1544,8 @@ VolSetInfo(struct rx_call *acid, afs_int32 atrans,
 	td->creationDate = astatus->creationDate;
     if (astatus->updateDate != -1)
 	td->updateDate = astatus->updateDate;
+    if (astatus->spare2 != -1)
+	td->volUpdateCounter = (unsigned int)astatus->spare2;
     VUpdateVolume(&error, tv);
     tt->rxCallPtr = (struct rx_call *)0;
     if (TRELE(tt))
@@ -1849,7 +1851,8 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid, afs_int32
 		(long)tv->header->diskstuff.weekUse[4] +
 		(long)tv->header->diskstuff.weekUse[5] +
 		(long)tv->header->diskstuff.weekUse[6];
-	    pntr->flags = pntr->spare2 = pntr->spare3 = (long)0;
+	    pntr->spare2 = V_volUpCounter(tv);
+	    pntr->flags = pntr->spare3 = (long)0;
 	    VDetachVolume(&error, tv);	/*free the volume */
 	    tv = (Volume *) 0;
 	    if (error) {
@@ -2171,7 +2174,7 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 		pntr->volid = volid;
 		goto drop;
 	    }
-	    tv = VAttachVolumeByName(&error, pname, volname, V_READONLY);
+	    tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
 	    if (error) {
 		pntr->status = 0;	/*things are messed up */
 		strcpy(pntr->name, volname);
@@ -2224,7 +2227,8 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 		(long)tv->header->diskstuff.weekUse[4] +
 		(long)tv->header->diskstuff.weekUse[5] +
 		(long)tv->header->diskstuff.weekUse[6];
-	    pntr->flags = pntr->spare2 = pntr->spare3 = (long)0;
+	    pntr->spare2 = V_volUpCounter(tv);
+	    pntr->flags = pntr->spare3 = (long)0;
 	    VDetachVolume(&error, tv);	/*free the volume */
 	    tv = (Volume *) 0;
 	    if (error) {
@@ -2413,7 +2417,7 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	    /*
 	     * Attach the volume, give up on this volume if we can't.
 	     */
-	    tv = VAttachVolumeByName(&error, pname, volname, V_READONLY);
+	    tv = VAttachVolumeByName(&error, pname, volname, V_PEEK);
 	    if (error) {
 		xInfoP->status = 0;	/*things are messed up */
 		strcpy(xInfoP->name, volname);
@@ -2640,7 +2644,7 @@ SAFSVolSetIdsTypes(struct rx_call *acid, afs_int32 atid, char name[], afs_int32
 
     code = VolSetIdsTypes(acid, atid, name, type, pId, cloneId, backupId);
     osi_auditU(acid, VS_SetIdTyEvent, code, AUD_LONG, atid, AUD_STR, name,
-	       AUD_STR, type, AUD_LONG, pId, AUD_LONG, cloneId, AUD_LONG,
+	       AUD_LONG, type, AUD_LONG, pId, AUD_LONG, cloneId, AUD_LONG,
 	       backupId, AUD_END);
     return code;
 }
